@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -32,13 +32,31 @@
 #ifndef PARSER_H
 #define PARSER_H
 
+#include <string>
+#include <vector>
+#include <set>
+#include "ticcutils/LogStream.h"
+#include "ticcutils/Configuration.h"
+#include "ticcutils/Unicode.h"
+#include "libfolia/folia.h"
+#include "ucto/tokenize.h"
+#include "timbl/TimblAPI.h"
+
 struct parseData;
+class TimerBlock;
 
 class Parser {
  public:
-  Parser(TiCC::LogStream* logstream):pairs(0),dir(0),rels(0),isInit(false) {
-    parseLog = new TiCC::LogStream(logstream, "parser-");
-  };
+  explicit Parser( TiCC::LogStream* logstream ):
+  pairs(0),
+    dir(0),
+    rels(0),
+    maxDepSpan( 0 ),
+    isInit( false ),
+    filter( 0 )
+      {
+	parseLog = new TiCC::LogStream(logstream, "parser-");
+      };
   ~Parser();
   bool init( const TiCC::Configuration& );
   void addDeclaration( folia::Document& doc ) const;
@@ -63,7 +81,9 @@ class Parser {
   std::string dep_tagset;
   std::string POS_tagset;
   std::string MWU_tagset;
-  Tokenizer::UnicodeFilter *filter;
+  std::string textclass;
+  TiCC::UniFilter *filter;
+  Parser( const Parser& ){}; // inhibit copies
 };
 
 
