@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -32,15 +32,16 @@
 #ifndef CGN_TAGGER_MOD_H
 #define CGN_TAGGER_MOD_H
 
-#include "frog/pos_tagger_mod.h"
+#include "frog/tagger_base.h"
 
-class CGNTagger: public POSTagger {
+class CGNTagger: public BaseTagger {
  public:
- CGNTagger( TiCC::LogStream *l ): POSTagger( l ){};
+  explicit CGNTagger( TiCC::LogStream *l ): BaseTagger( l, "tagger" ){};
   bool init( const TiCC::Configuration& );
-  void Classify( const std::vector<folia::Word *>& );
-  void post_process( const std::vector<folia::Word *>& words );
+  void addDeclaration( folia::Document& ) const;
+  void post_process( const std::vector<folia::Word*>& );
  private:
+  void addTag( folia::Word *, const std::string&, double );
   void fillSubSetTable();
   std::string getSubSet( const std::string& , const std::string& );
   std::multimap<std::string,std::string> cgnSubSets;
