@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -32,6 +32,10 @@
 #ifndef CLEX_H
 #define CLEX_H
 
+#include <iostream>
+#include <string>
+#include <map>
+
 namespace CLEX {
   enum Type { UNASS, N, A, Q, V, D, O, B, P, C, I, X, Z, PN, AFFIX, XAFFIX,
 	      GLUE, NEUTRAL };
@@ -42,8 +46,8 @@ namespace CLEX {
   Type toCLEX( const char );
   extern std::map<CLEX::Type,std::string> tagNames;
   extern std::map<char,std::string> iNames;
-  inline std::string get_iDescr( char c ) { return iNames[c]; }
-  inline std::string get_tDescr( CLEX::Type t ) {  return tagNames[t]; }
+  inline const std::string& get_iDescr( char c ) { return iNames[c]; }
+  inline const std::string& get_tDescr( CLEX::Type t ) {  return tagNames[t]; }
 }
 
 std::ostream& operator<<( std::ostream&, const CLEX::Type& );
