@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -32,29 +32,24 @@
 #ifndef IOB_TAGGER_MOD_H
 #define IOB_TAGGER_MOD_H
 
-class IOBTagger {
+#include "frog/tagger_base.h"
+
+class IOBTagger: public BaseTagger {
  public:
-  IOBTagger(TiCC::LogStream *);
-  ~IOBTagger();
+  explicit IOBTagger( TiCC::LogStream *l ): BaseTagger( l, "IOB" ){};
   bool init( const TiCC::Configuration& );
   void addDeclaration( folia::Document& ) const;
-  void Classify( const std::vector<folia::Word *>& );
-  std::string getTagset() const { return tagset; };
-  std::string set_eos_mark( const std::string& );
+  void Classify( const std::vector<folia::Word*>& );
+  void post_process( const std::vector<folia::Word*>& );
  private:
   void addChunk( folia::ChunkingLayer *,
 		 const std::vector<folia::Word*>&,
 		 const std::vector<double>&,
+		 const std::string&,
 		 const std::string& );
   void addIOBTags( const std::vector<folia::Word*>&,
 		   const std::vector<std::string>&,
 		   const std::vector<double>& );
-  MbtAPI *tagger;
-  TiCC::LogStream *iobLog;
-  int debug;
-  std::string version;
-  std::string tagset;
-  Tokenizer::UnicodeFilter *filter;
 };
 
 #endif // IOB_TAGGER_MOD_H
