@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -32,6 +32,12 @@
 #ifndef MBMA_BRACKETS_H
 #define MBMA_BRACKETS_H
 
+#include <vector>
+#include <list>
+#include "unicode/unistr.h"
+#include "ticcutils/LogStream.h"
+#include "frog/clex.h"
+
 enum Status { INFO, PARTICLE, PARTICIPLE, STEM, COMPLEX, INFLECTION,
 	      DERIVATIONAL, FAILED };
 
@@ -55,28 +61,31 @@ namespace folia {
 class RulePart;
 
 class BaseBracket {
-public:
+ public:
  BaseBracket( CLEX::Type t, const std::vector<CLEX::Type>& R, int flag,
 	      TiCC::LogStream& l ):
   RightHand(R),
     cls(t),
+    _status( FAILED ),
     debugFlag(flag),
     myLog(l)
-
-   {};
+    {};
  BaseBracket( CLEX::Type t, int flag, TiCC::LogStream& l ):
-  cls(t), debugFlag(flag), myLog(l)
-  {};
+  cls(t),
+    _status( FAILED ),
+    debugFlag(flag),
+    myLog(l)
+    {};
   virtual ~BaseBracket() {};
 
   Status status() const { return _status; };
   void set_status( const Status s ) { _status = s; };
-  virtual UnicodeString morpheme() const { return "";};
+  virtual icu::UnicodeString morpheme() const { return "";};
   virtual std::string inflection() const { return ""; };
   virtual std::string original() const { return ""; };
   virtual int infixpos() const { return -1; };
   virtual bool isglue() const { return false; };
-  virtual UnicodeString put( bool = true ) const;
+  virtual icu::UnicodeString put( bool = true ) const;
   virtual BaseBracket *append( BaseBracket * ){ abort(); };
   virtual bool isNested() { return false; };
   virtual void resolveGlue(){ abort(); };
@@ -102,9 +111,9 @@ public:
 class BracketLeaf: public BaseBracket {
 public:
   BracketLeaf( const RulePart&, int, TiCC::LogStream& );
-  BracketLeaf( CLEX::Type, const UnicodeString&, int, TiCC::LogStream& );
-  UnicodeString put( bool = true ) const;
-  UnicodeString morpheme() const { return morph; };
+  BracketLeaf( CLEX::Type, const icu::UnicodeString&, int, TiCC::LogStream& );
+  icu::UnicodeString put( bool = true ) const;
+  icu::UnicodeString morpheme() const { return morph; };
   std::string inflection() const { return inflect; };
   std::string original() const { return orig; };
   int infixpos() const { return ifpos; };
@@ -115,7 +124,7 @@ public:
 private:
   int ifpos;
   bool glue;
-  UnicodeString morph;
+  icu::UnicodeString morph;
   std::string orig;
   std::string inflect;
 };
@@ -127,7 +136,7 @@ class BracketNest: public BaseBracket {
   ~BracketNest();
   bool isNested() { return true; };
   void clearEmptyNodes();
-  UnicodeString put( bool = true ) const;
+  icu::UnicodeString put( bool = true ) const;
   bool testMatch( std::list<BaseBracket*>& result,
 		  const std::list<BaseBracket*>::iterator& rpos,
 		  std::list<BaseBracket*>::iterator& bpos );
