@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -32,8 +32,14 @@
 #ifndef MBMA_MOD_H
 #define MBMA_MOD_H
 
+#include <map>
+#include <vector>
+#include <list>
 #include <unicode/translit.h>
 #include "ticcutils/LogStream.h"
+#include "ticcutils/Unicode.h"
+#include "ticcutils/Configuration.h"
+#include "libfolia/folia.h"
 #include "frog/clex.h"
 #include "frog/mbma_rule.h"
 #include "frog/mbma_brackets.h"
@@ -43,29 +49,14 @@ namespace Timbl{
   class TimblAPI;
 }
 
-namespace Tokenizer {
-  class UnicodeFilter;
-}
-
-namespace TiCC {
-  class Configuration;
-}
-
-namespace folia {
-  class Word;
-  class Document;
-  class MorphologyLayer;
-  class Morpheme;
-}
-
 class Mbma {
  public:
-  Mbma(TiCC::LogStream *);
+ explicit Mbma( TiCC::LogStream * );
   ~Mbma();
   bool init( const TiCC::Configuration& );
   void addDeclaration( folia::Document& doc ) const;
   void Classify( folia::Word * );
-  void Classify( const UnicodeString& );
+  void Classify( const icu::UnicodeString& );
   void filterHeadTag( const std::string& );
   void filterSubTags( const std::vector<std::string>& );
   void assign_compounds();
@@ -73,8 +64,8 @@ class Mbma {
   std::vector<std::pair<std::string,std::string>> getResults( ) const;
   void setDeepMorph( bool b ){ doDeepMorph = b; };
   void clearAnalysis();
-  Rule* matchRule( const std::vector<std::string>&, const UnicodeString& );
-  std::vector<Rule*> execute( const UnicodeString& ,
+  Rule* matchRule( const std::vector<std::string>&, const icu::UnicodeString& );
+  std::vector<Rule*> execute( const icu::UnicodeString& ,
 			      const std::vector<std::string>& );
   static std::map<std::string,std::string> TAGconv;
   static std::string mbma_tagset;
@@ -85,10 +76,8 @@ class Mbma {
   bool readsettings( const std::string&, const std::string& );
   void fillMaps();
   void init_cgn( const std::string&, const std::string& );
-  Transliterator * init_trans();
-  UnicodeString filterDiacritics( const UnicodeString& ) const;
-  void getFoLiAResult( folia::Word *, const UnicodeString& ) const;
-  std::vector<std::string> make_instances( const UnicodeString& word );
+  void getFoLiAResult( folia::Word *, const icu::UnicodeString& ) const;
+  std::vector<std::string> make_instances( const icu::UnicodeString& word );
   CLEX::Type getFinalTag( const std::list<BaseBracket*>& );
   int debugFlag;
   void addMorph( folia::MorphologyLayer *,
@@ -104,9 +93,10 @@ class Mbma {
   Timbl::TimblAPI *MTree;
   std::vector<Rule*> analysis;
   std::string version;
+  std::string textclass;
   TiCC::LogStream *mbmaLog;
-  Transliterator *transliterator;
-  Tokenizer::UnicodeFilter *filter;
+  TiCC::UniFilter *filter;
+  bool filter_diac;
   bool doDeepMorph;
 };
 
