@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -32,6 +32,13 @@
 #ifndef MWU_CHUNKER_H
 #define MWU_CHUNKER_H
 
+#include <ostream>
+#include <string>
+#include "ticcutils/LogStream.h"
+#include "ticcutils/Configuration.h"
+#include "ticcutils/Unicode.h"
+#include "libfolia/folia.h"
+
 class mwuAna {
   friend std::ostream& operator<< (std::ostream&, const mwuAna& );
  public:
@@ -46,6 +53,7 @@ class mwuAna {
 
   bool isSpec(){ return spec; };
   folia::EntitiesLayer *addEntity( const std::string&,
+				   const std::string&,
 				   folia::Sentence *,
 				   folia::EntitiesLayer * );
 
@@ -61,7 +69,7 @@ class mwuAna {
 class Mwu {
   friend std::ostream& operator<< (std::ostream&, const Mwu& );
  public:
-  Mwu(TiCC::LogStream*);
+  explicit Mwu(TiCC::LogStream*);
   ~Mwu();
   void reset();
   bool init( const TiCC::Configuration& );
@@ -80,9 +88,10 @@ class Mwu {
   mymap2 MWUs;
   TiCC::LogStream *mwuLog;
   std::string version;
+  std::string textclass;
   std::string mwu_tagset;
   std::string glue_tag;
-  Tokenizer::UnicodeFilter *filter;
+  TiCC::UniFilter *filter;
 };
 
 #endif
