@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -28,11 +28,13 @@
 #ifndef UCTO_TOKENIZER_MOD_H
 #define UCTO_TOKENIZER_MOD_H
 
+#include "libfolia/folia.h"
 #include "ucto/tokenize.h"
+#include "ticcutils/Configuration.h"
 
 class UctoTokenizer {
  public:
-  UctoTokenizer(TiCC::LogStream *);
+  explicit UctoTokenizer( TiCC::LogStream * );
   ~UctoTokenizer() { delete tokenizer; };
   bool init( const TiCC::Configuration& );
   void setUttMarker( const std::string& );
@@ -42,8 +44,10 @@ class UctoTokenizer {
   void setInputEncoding( const std::string& );
   void setQuoteDetection( bool );
   void setInputXml( bool );
-  void setTextClass( const std::string& );
+  void setInputClass( const std::string& );
+  void setOutputClass( const std::string& );
   void setDocID( const std::string& );
+  void setTextRedundancy( const std::string& );
   folia::Document *tokenizestring( const std::string& );
   folia::Document *tokenize( std::istream& );
   bool tokenize( folia::Document& );
