@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -50,19 +50,17 @@
 
 // individual module headers
 
-#include "frog/Frog.h" //internal interface, included by all modules
+#include "frog/Frog-util.h"
 #include "frog/FrogAPI.h" //public API interface
 #include "frog/ucto_tokenizer_mod.h"
-#include "frog/pos_tagger_mod.h"
+#include "frog/tagger_base.h"
 #include "ticcutils/StringOps.h"
 #include "ticcutils/CommandLine.h"
 #include "ticcutils/FileUtils.h"
 
 using namespace std;
-using namespace folia;
-using namespace TiCC;
 
-#define LOG *Log(theErrLog)
+#define LOG *TiCC::Log(theErrLog)
 
 string testDirName;
 string outputFileName;
@@ -91,7 +89,7 @@ set<string> fileNames;
 
 */
 
-Configuration configuration;
+TiCC::Configuration configuration;
 
 void usage( ) {
   cout << endl << "Options:\n";
@@ -99,22 +97,32 @@ void usage( ) {
        << "\t -e <encoding>          specify encoding of the input (default UTF-8)\n"
        << "\t -t <testfile>          Run frog on this file\n"
        << "\t -x <testfile>          Run frog on this FoLiA XML file. Or the files from 'testdir'\n"
-       << "\t --textclass=<cls>      use the specified class to search for text in the the FoLia docs.\n"
+       << "\t --textclass=<cls>      use the specified class to search for text in the the FoLiA docs. (default 'current').\n"
+       << "\t\t\t the same value is used for output too.\n"
+       << "\t --inputclass=<cls>     use the specified class to search for text in the the FoLiA docs. (default 'current') \n"
+       << "\t --outputclass=<cls>    use the specified class to output text in the the FoLia docs. (default 'inputclass') \n"
        << "\t --testdir=<directory>  All files in this dir will be tested\n"
        << "\t --uttmarker=<mark>     utterances are separated by 'mark' symbols"
        << "\t                        (default none)\n"
        << "\t -n                     Assume input file to hold one sentence per line\n"
+       << "\t --retry                assume frog is running again on the same input,\n"
+       << "\t                        already done files are skipped. (detected on the basis of already existing output files)\n"
        << "\t --max-parser-tokens=<n> inhibit parsing when a sentence contains over 'n' tokens. (default: 500, needs already 16Gb of memory!)\n"
        << "\t -Q                   Enable quote detection in tokeniser.\n"
+       << "\t-T or --textredundancy=[full|minimal|none]  - set text redundancy level in the tokenizer for text nodes in FoLiA output: " << endl
+       << "\t                    'full' - add text to all levels: <p> <s> <w> etc." << endl
+       << "\t                    'minimal' - don't introduce text on higher levels, but retain what is already there." << endl
+       << "\t                    'none' - only introduce text on <w>, AND remove all text from higher levels" << endl
        << "\t============= MODULE SELECTION ==========================================\n"
        << "\t --skip=[mptncla]    Skip Tokenizer (t), Lemmatizer (l), Morphological Analyzer (a), Chunker (c), Multi-Word Units (m), Named Entity Recognition (n), or Parser (p) \n"
        << "\t============= CONFIGURATION OPTIONS =====================================\n"
-       << "\t -c <filename>    Set configuration file (default "
-       << FrogAPI::defaultConfigFile() << ")\n"
+       << "\t -c <filename>    Set configuration file (default " << FrogAPI::defaultConfigFile() << ")\n"
+       << "\t --override <section>.<parameter>=<value>    Override a configuration option, can be used multiple times\n"
        << "\t --language <language-list>  Set the languages. e.g. --language=nld,eng,por"
        << "\t\t The first language in the list will be the default. (default dutch).\n"
        << "\t============= OUTPUT OPTIONS ============================================\n"
        << "\t -o <outputfile>	    Output columned output to file, instead of default stdout\n"
+       << "\t --nostdout            suppress columned output to stdout\n"
        << "\t -X <xmlfile>          Output also to an XML file in FoLiA format\n"
        << "\t --id=<docid>          Document ID, used in FoLiA output. (Default 'untitled')\n"
        << "\t --outputdir=<dir>     Output to dir, instead of default stdout\n"
@@ -126,16 +134,16 @@ void usage( ) {
        << "\t -d <debug level>  (for more verbosity)\n"
        << "\t --debug=<module><level>,<module><level>... (eg --debug=l5,n3) \n"
        << "\t\t Set debug value for Tokenizer (t), Lemmatizer (l), Morphological Analyzer (a), Chunker (c), Multi-Word Units (m), Named Entity Recognition (n), or Parser (p) \n"
-       << "\t -S <port>              Run as server instead of reading from testfile\n"
+       << "\t -S <port>             Run as server instead of reading from testfile\n"
 #ifdef HAVE_OPENMP
-       << "\t --threads=<n>       Use a maximum of 'n' threads. Default: 8. \n"
+       << "\t --threads=<n>         Use a maximum of 'n' threads. Default: 8. \n"
 #endif
-       << "\t                     (but always 1 for server mode)\n";
+       << "\t                       (but always 1 for server mode)\n";
 }
 
 bool parse_args( TiCC::CL_Options& Opts,
 		 FrogOptions& options,
-		 LogStream* theErrLog ) {
+		 TiCC::LogStream* theErrLog ) {
   // process the command line and fill FrogOptions to initialize the API
   // also fill some globals we use for our own main.
 
@@ -151,15 +159,14 @@ bool parse_args( TiCC::CL_Options& Opts,
     language = "none";
   }
   else {
-    vector<string> lang;
-    int num = split_at( languages, lang, "," );
-    if ( num < 0 ){
+    vector<string> lang_v = TiCC::split_at( languages, "," );
+    if ( lang_v.empty() ){
       cerr<< "invalid value in --languages=" << languages
 	  << " option. " << endl;
       return false;
     }
-    language = lang[0]; // the first mentioned is the default.
-    if ( num > 1 ){
+    language = lang_v[0]; // the first mentioned is the default.
+    if ( lang_v.size() > 1 ){
       cerr << "WARNING: you used the --language=" << languages << " option"
 	   << " with more then one language " << endl
 	   << "\t specified. These values will be handled to the tokenizer,"
@@ -198,7 +205,7 @@ bool parse_args( TiCC::CL_Options& Opts,
   string value;
   // debug opts
   if ( Opts.extract ('d', value) ) {
-    if ( !stringTo<int>( value, options.debugFlag ) ){
+    if ( !TiCC::stringTo<int>( value, options.debugFlag ) ){
       LOG << "-d value should be an integer" << endl;
       return false;
     }
@@ -209,13 +216,12 @@ bool parse_args( TiCC::CL_Options& Opts,
   }
   if ( Opts.extract( "debug", value ) ) {
     value = TiCC::lowercase( value );
-    vector<string> vec;
-    TiCC::split_at( value, vec, "," );
+    vector<string> vec = TiCC::split_at( value, "," );
     for ( const auto& val : vec ){
       char mod = val[0];
       string value = val.substr(1);
       int dbval = 0;
-      if ( !stringTo<int>( value, dbval ) ){
+      if ( !TiCC::stringTo<int>( value, dbval ) ){
 	cerr << "expected integer value for --debug=" << mod << value << endl;
 	return false;
       }
@@ -247,7 +253,18 @@ bool parse_args( TiCC::CL_Options& Opts,
       }
     }
   }
-
+  string redundancy;
+  Opts.extract( 'T', redundancy );
+  Opts.extract( "textredundancy", redundancy );
+  if ( !redundancy.empty() ){
+    if ( redundancy != "full"
+	 && redundancy != "minimal"
+	 && redundancy != "none" ){
+      LOG << "unknown textredundancy level: " << redundancy << endl;
+      return false;
+    }
+    options.textredundancy = redundancy;
+  }
   options.doSentencePerLine = Opts.extract( 'n' );
   options.doQuoteDetection = Opts.extract( 'Q' );
   if ( Opts.extract( "skip", value )) {
@@ -277,16 +294,21 @@ bool parse_args( TiCC::CL_Options& Opts,
     options.doDeepMorph = true;
     options.doMorph = true;
   }
-
+  options.doRetry = Opts.extract( "retry" );
+  options.noStdOut = Opts.extract( "nostdout" );
   Opts.extract( 'e', options.encoding );
 
   if ( Opts.extract( "max-parser-tokens", value ) ){
-    if ( !stringTo<unsigned int>( value, options.maxParserTokens ) ){
+    if ( !TiCC::stringTo<unsigned int>( value, options.maxParserTokens ) ){
       LOG << "max-parser-tokens value should be an integer" << endl;
       return false;
     }
   }
 
+  if ( Opts.extract( "ner-override", value ) ){
+    configuration.setatt( "ner_override", value, "NER" );
+  }
+
   options.doServer = Opts.extract('S', options.listenport );
 
 #ifdef HAVE_OPENMP
@@ -297,7 +319,7 @@ bool parse_args( TiCC::CL_Options& Opts,
   }
   else if ( Opts.extract( "threads", value ) ){
     int num;
-    if ( !stringTo<int>( value, num ) || num < 1 ){
+    if ( !TiCC::stringTo<int>( value, num ) || num < 1 ){
       LOG << "threads value should be a positive integer" << endl;
       return false;
     }
@@ -382,13 +404,36 @@ bool parse_args( TiCC::CL_Options& Opts,
       }
     }
   }
-  if ( Opts.extract( "textclass", options.textclass ) ){
-    if ( !options.doXMLin ){
-      LOG << "--textclass is only valid when -x is also present" << endl;
+  string textclass;
+  string inputclass;
+  string outputclass;
+  Opts.extract( "textclass", textclass );
+  Opts.extract( "inputclass", inputclass );
+  Opts.extract( "outputclass", outputclass );
+  if ( !textclass.empty() ){
+    if ( !inputclass.empty() || !outputclass.empty() ){
+      LOG << "when --textclass is specified, --inputclass or --outputclass may NOT be present." << endl;
       return false;
     }
+    options.inputclass = textclass;
+    options.outputclass = textclass;
+    configuration.setatt( "inputclass", textclass );
+    configuration.setatt( "outputclass", textclass );
+  }
+  else {
+    if ( !inputclass.empty() ){
+      options.inputclass = inputclass;
+      configuration.setatt( "inputclass", inputclass );
+      if ( outputclass.empty() ){
+	options.outputclass = inputclass;
+	configuration.setatt( "outputclass", inputclass );
+      }
+    }
+    if ( !outputclass.empty() ){
+      options.outputclass = outputclass;
+      configuration.setatt( "outputclass", outputclass );
+    }
   }
-
   if ( !XMLoutFileName.empty() && !testDirName.empty() ){
     LOG << "useless -X value" << endl;
     return false;
@@ -424,6 +469,26 @@ bool parse_args( TiCC::CL_Options& Opts,
       return false;
     }
   }
+  string overridestatement;
+  while ( Opts.extract("override", overridestatement )) {
+    vector<string> values;
+    const int num = TiCC::split_at( overridestatement,values,  "=" );
+    if ( num == 2 ) {
+        vector<string> module_param;
+        const int num2 = TiCC::split_at(values[0], module_param, "." );
+        if (num2 == 2) {
+            LOG << "Overriding configuration parameter " << module_param[0] << "." << module_param[1] << " with " << values[1] << endl;
+            configuration.setatt( module_param[1] , values[1], module_param[0] );
+        } else if (num2 == 1) {
+            LOG << "Overriding configuration parameter " << module_param[0] << " with " << values[1] << endl;
+            configuration.setatt( module_param[0] , values[1]);
+        } else {
+            LOG << "Invalid syntax for --override option" << endl;
+        }
+    } else {
+        LOG << "Invalid syntax for --override option" << endl;
+    }
+  }
   if ( !Opts.empty() ){
     LOG << "unhandled commandline options: " << Opts.toString() << endl;
 
@@ -432,11 +497,11 @@ bool parse_args( TiCC::CL_Options& Opts,
   return true;
 }
 
-bool StillRunning = true;
+static bool StillRunning = true;
 
 void KillServerFun( int Signal ){
   if ( Signal == SIGTERM ){
-    cerr << "KillServerFun caught a signal SIGTERM" << endl;
+    cerr << TiCC::Timer::now() << " KillServerFun caught a signal SIGTERM" << endl;
     sleep(5); // give children some spare time...
     StillRunning = false;
   }
@@ -444,7 +509,7 @@ void KillServerFun( int Signal ){
 
 
 int main(int argc, char *argv[]) {
-  cerr << "frog " << VERSION << " (c) CLTS, ILK 1998 - 2017" << endl
+  cerr << "frog " << VERSION << " (c) CLTS, ILK 1998 - 2018" << endl
        << "CLST  - Centre for Language and Speech Technology,"
        << "Radboud University" << endl
        << "ILK   - Induction of Linguistic Knowledge Research Group,"
@@ -454,16 +519,19 @@ int main(int argc, char *argv[]) {
        << Timbl::VersionName() << ", "
        << TimblServer::VersionName() << ", "
        << Tagger::VersionName() << "]" << endl;
-  LogStream *theErrLog = new LogStream( cerr, "frog-", StampMessage );
+  TiCC::LogStream *theErrLog
+    = new TiCC::LogStream( cerr, "frog-", StampMessage );
   std::ios_base::sync_with_stdio(false);
   FrogOptions options;
 
   try {
-    TiCC::CL_Options Opts("c:e:o:t:x::X::nQhVd:S:",
-			  "textclass:,testdir:,uttmarker:,max-parser-tokens:,"
+    TiCC::CL_Options Opts("c:e:o:t:T:x::X::nQhVd:S:",
+			  "textclass:,inputclass:,outputclass:,testdir:,"
+			  "uttmarker:,max-parser-tokens:,textredundancy:,"
 			  "skip:,id:,outputdir:,xmldir:,tmpdir:,deep-morph,"
-			  "help,language:,"
-			  "debug:,keep-parser-files,version,threads:,KANON");
+			  "help,language:,retry,nostdout,ner-override:,"
+			  "debug:,keep-parser-files,version,threads:,"
+			  "override:,KANON");
     Opts.init(argc, argv);
     if ( Opts.is_present('V' ) || Opts.is_present("version" ) ){
       // we already did show what we wanted.
@@ -479,25 +547,31 @@ int main(int argc, char *argv[]) {
       throw runtime_error( "init failed" );
     }
     FrogAPI frog( options, configuration, theErrLog );
-
     if ( !fileNames.empty() ) {
       string outPath = outputDirName;
       string xmlPath = xmlDirName;
 
       ostream *outS = 0;
       if ( !outputFileName.empty() ){
+	if ( options.doRetry && TiCC::isFile( outputFileName ) ){
+	  LOG << "retry, skip: " << outputFileName << " already exists" << endl;
+	  return EXIT_SUCCESS;
+	}
 	if ( !TiCC::createPath( outputFileName ) ) {
 	  LOG << "problem: unable to create outputfile: "
-			  << outputFileName << endl;
+	      << outputFileName << endl;
 	  return EXIT_FAILURE;
 	}
         outS = new ofstream( outputFileName );
       }
+      if ( fileNames.size() > 1 ){
+	LOG << "start procesessing " << fileNames.size() << " files..." << endl;
+      }
       for ( auto const& name : fileNames ){
 	string testName = testDirName + name;
 	if ( !TiCC::isFile( testName ) ){
 	  LOG << "skip " << testName << " (file not found )"
-			  << endl;
+	      << endl;
 	  continue;
 	}
 	string outName;
@@ -510,6 +584,10 @@ int main(int argc, char *argv[]) {
 	    else {
 	      outName = outPath + name + ".out";
 	    }
+	    if ( options.doRetry && TiCC::isFile( outName ) ){
+	      LOG << "retry, skip: " << outName << " already exists" << endl;
+	      continue;
+	    }
 	    if ( !TiCC::createPath( outName ) ) {
 	      LOG << "problem frogging: " << name << endl
 			      << "unable to create outputfile: " << outName
@@ -517,7 +595,8 @@ int main(int argc, char *argv[]) {
 	      continue;
 	    }
 	    outS = new ofstream( outName );
-	  } else {
+	  }
+	  else {
 	    outS = &cout;
 	  }
 	}
@@ -532,13 +611,22 @@ int main(int argc, char *argv[]) {
 	  else if ( options.doXMLout )
 	    xmlOutName = name + ".xml"; // do not clobber the inputdir!
 	}
-	if ( !xmlOutName.empty() && !TiCC::createPath( xmlOutName ) ){
-	  LOG << "problem frogging: " << name << endl
-			  << "unable to create outputfile: " << xmlOutName
-			  << endl;
-	  continue;
+	if ( !xmlOutName.empty() ){
+	  if ( options.doRetry && TiCC::isFile( xmlOutName ) ){
+	    LOG << "retry, skip: " << xmlOutName << " already exists" << endl;
+	    continue;
+	  }
+	  if ( !TiCC::createPath( xmlOutName ) ){
+	    LOG << "problem frogging: " << name << endl
+		<< "unable to create outputfile: " << xmlOutName
+		<< endl;
+	    continue;
+	  }
+	  else {
+	    remove( xmlOutName.c_str() );
+	  }
 	}
-	LOG << "Frogging " << testName << endl;
+	LOG << TiCC::Timer::now() << " Frogging " << testName << endl;
 	try {
 	  frog.FrogFile( testName, *outS, xmlOutName );
 	}
@@ -549,14 +637,20 @@ int main(int argc, char *argv[]) {
 	}
 	if ( !outName.empty() ){
 	  LOG << "results stored in " << outName << endl;
-	  delete outS;
-	  outS = 0;
+	  if ( outS != &cout ){
+	    delete outS;
+	    outS = 0;
+	  }
 	}
       }
       if ( !outputFileName.empty() ){
 	LOG << "results stored in " << outputFileName << endl;
-	delete outS;
+	if ( outS != &cout ){
+	  delete outS;
+	  outS = 0;
+	}
       }
+      LOG << TiCC::Timer::now() << " Frog finished" << endl;
     }
     else if ( options.doServer ) {
       //first set up some things to deal with zombies
@@ -605,8 +699,9 @@ int main(int argc, char *argv[]) {
 	    throw( runtime_error( "Accept failed" ) );
 	  }
 	}
+	LOG << TiCC::Timer::now() << " server terminated by SIGTERM" << endl;
       }
-      catch ( std::exception& e ) {
+      catch ( exception& e ) {
 	LOG << "Server error:" << e.what() << " Exiting." << endl;
 	throw;
       }
