@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -28,10 +28,11 @@
       lamasoftware (at ) science.ru.nl
 */
 
+#include "frog/FrogAPI.h"
+
 #include <cstdlib>
 #include <cstdio>
 #include <unistd.h>
-#include <sys/wait.h>
 #include <signal.h>
 #include <string>
 #include <cstring>
@@ -39,6 +40,7 @@
 #include <sstream>
 #include <fstream>
 #include <vector>
+#include <unordered_map>
 #include "config.h"
 #ifdef HAVE_OPENMP
 #include <omp.h>
@@ -64,9 +66,10 @@
 #  endif /* defined(HAVE_READLINE_HISTORY_H) */
 #endif /* HAVE_READLINE_HISTORY */
 
+#include "ticcutils/PrettyPrint.h"
 
 // individual module headers
-#include "frog/FrogAPI.h" //will also include Frog.h (internals), FrogAPI.h is for public  interface
+#include "frog/Frog-util.h"
 #include "frog/ucto_tokenizer_mod.h"
 #include "frog/mblem_mod.h"
 #include "frog/mbma_mod.h"
@@ -78,11 +81,10 @@
 
 
 using namespace std;
-using namespace folia;
-using namespace TiCC;
 using namespace Tagger;
+using TiCC::operator<<;
 
-#define LOG *Log(theErrLog)
+#define LOG *TiCC::Log(theErrLog)
 
 string configDir = string(SYSCONF_PATH) + "/" + PACKAGE + "/";
 string configFileName = configDir + "frog.cfg";
@@ -106,6 +108,8 @@ FrogOptions::FrogOptions() {
   doSentencePerLine = false;
   doQuoteDetection = false;
   doDirTest = false;
+  doRetry = false;
+  noStdOut = false;
   doServer = false;
   doXMLin =  false;
   doXMLout =  false;
@@ -123,12 +127,56 @@ FrogOptions::FrogOptions() {
   uttmark = "<utt>";
   listenport = "void";
   docid = "untitled";
+  inputclass="current";
+  outputclass="current";
+  textredundancy="minimal";
   debugFlag = 0;
 }
 
+void FrogAPI::test_version( const string& where, double minimum ){
+  string version = configuration.lookUp( "version", where );
+  double v = 0.0;
+  if ( !version.empty() ){
+    if ( !TiCC::stringTo( version, v ) ){
+      v = 0.5;
+    }
+  }
+  if ( where == "IOB" ){
+    if ( v < minimum ){
+      LOG << "[[" << where << "]] Wrong FrogData!. "
+	  << "Expected version " << minimum << " or higher for module: "
+	  << where << endl;
+      if ( version.empty() ) {
+	LOG << "but no version info was found!." << endl;
+      }
+      else {
+	LOG << "but found version " << v << endl;
+      }
+      throw runtime_error( "Frog initialization failed" );
+    }
+  }
+  else if ( where == "NER" ){
+    if ( v < minimum ){
+      LOG << "[[" << where << "]] Wrong FrogData!. "
+	  << "Expected version " << minimum << " or higher for module: "
+	  << where << endl;
+      if ( version.empty() ) {
+	LOG << "but no version info was found!." << endl;
+      }
+      else {
+	LOG << "but found version " << v << endl;
+      }
+      throw runtime_error( "Frog initialization failed" );
+    }
+  }
+  else {
+    throw logic_error( "unknown where:" + where );
+  }
+}
+
 FrogAPI::FrogAPI( FrogOptions &opt,
-		  const Configuration &conf,
-		  LogStream *log ):
+		  const TiCC::Configuration &conf,
+		  TiCC::LogStream *log ):
   configuration(conf),
   options(opt),
   theErrLog(log),
@@ -136,7 +184,7 @@ FrogAPI::FrogAPI( FrogOptions &opt,
   myMblem(0),
   myMwu(0),
   myParser(0),
-  myPoSTagger(0),
+  myCGNTagger(0),
   myIOBTagger(0),
   myNERTagger(0),
   tokenizer(0)
@@ -144,6 +192,11 @@ FrogAPI::FrogAPI( FrogOptions &opt,
   // for some modules init can take a long time
   // so first make sure it will not fail on some trivialities
   //
+  if ( options.doTok && !configuration.hasSection("tagger") ){
+    LOG << "Missing [[tagger]] section in config file." << endl;
+    LOG << "cannot run without a tagger!" << endl;
+    throw runtime_error( "Frog initialization failed" );
+  }
   if ( options.doTok && !configuration.hasSection("tokenizer") ){
     LOG << "Missing [[tokenizer]] section in config file." << endl;
     LOG << "Disabled the tokenizer." << endl;
@@ -154,15 +207,25 @@ FrogAPI::FrogAPI( FrogOptions &opt,
     LOG << "Disabled the Morhological analyzer." << endl;
     options.doMorph = false;
   }
-  if ( options.doIOB && !configuration.hasSection("IOB") ){
-    LOG << "Missing [[IOB]] section in config file." << endl;
-    LOG << "Disabled the IOB Chunker." << endl;
-    options.doIOB = false;
+  if ( options.doIOB ){
+    if ( !configuration.hasSection("IOB") ){
+      LOG << "Missing [[IOB]] section in config file." << endl;
+      LOG << "Disabled the IOB Chunker." << endl;
+      options.doIOB = false;
+    }
+    else {
+      test_version( "IOB", 2.0 );
+    }
   }
-  if ( options.doNER && !configuration.hasSection("NER") ){
-    LOG << "Missing [[NER]] section in config file." << endl;
-    LOG << "Disabled the NER." << endl;
-    options.doNER = false;
+  if ( options.doNER ) {
+    if ( !configuration.hasSection("NER") ){
+      LOG << "Missing [[NER]] section in config file." << endl;
+      LOG << "Disabled the NER." << endl;
+      options.doNER = false;
+    }
+    else {
+      test_version( "NER", 2.0 );
+    }
   }
   if ( options.doMwu && !configuration.hasSection("mwu") ){
     LOG << "Missing [[mwu]] section in config file." << endl;
@@ -191,20 +254,22 @@ FrogAPI::FrogAPI( FrogOptions &opt,
       tokenizer->setInputEncoding( options.encoding );
       tokenizer->setInputXml( options.doXMLin );
       tokenizer->setUttMarker( options.uttmark );
-      tokenizer->setTextClass( options.textclass );
-      myPoSTagger = new CGNTagger(theErrLog);
-      stat = myPoSTagger->init( configuration );
+      tokenizer->setInputClass( options.inputclass );
+      tokenizer->setOutputClass( options.outputclass );
+      tokenizer->setTextRedundancy( options.textredundancy );
+      myCGNTagger = new CGNTagger( theErrLog );
+      stat = myCGNTagger->init( configuration );
       if ( stat ){
-	myPoSTagger->set_eos_mark( options.uttmark );
+	myCGNTagger->set_eos_mark( options.uttmark );
 	if ( options.doIOB ){
-	  myIOBTagger = new IOBTagger(theErrLog);
+	  myIOBTagger = new IOBTagger( theErrLog );
 	  stat = myIOBTagger->init( configuration );
 	  if ( stat ){
 	    myIOBTagger->set_eos_mark( options.uttmark );
 	  }
 	}
 	if ( stat && options.doNER ){
-	  myNERTagger = new NERTagger(theErrLog);
+	  myNERTagger = new NERTagger( theErrLog );
 	  stat = myNERTagger->init( configuration );
 	  if ( stat ){
 	    myNERTagger->set_eos_mark( options.uttmark );
@@ -254,20 +319,34 @@ FrogAPI::FrogAPI( FrogOptions &opt,
 #endif
 
     bool tokStat = true;
+    string tokWhat;
     bool lemStat = true;
+    string lemWhat;
     bool mwuStat = true;
+    string mwuWhat;
     bool mbaStat = true;
+    string mbaWhat;
     bool parStat = true;
+    string parWhat;
     bool tagStat = true;
+    string tagWhat;
     bool iobStat = true;
+    string iobWhat;
     bool nerStat = true;
+    string nerWhat;
 
 #pragma omp parallel sections
     {
 #pragma omp section
       {
-	tokenizer = new UctoTokenizer(theErrLog);
-	tokStat = tokenizer->init( configuration );
+	try {
+	  tokenizer = new UctoTokenizer(theErrLog);
+	  tokStat = tokenizer->init( configuration );
+	}
+	catch ( const exception& e ){
+	  tokWhat = e.what();
+	  tokStat = false;
+	}
 	if ( tokStat ){
 	  tokenizer->setPassThru( !options.doTok );
 	  tokenizer->setDocID( options.docid );
@@ -276,56 +355,100 @@ FrogAPI::FrogAPI( FrogOptions &opt,
 	  tokenizer->setInputEncoding( options.encoding );
 	  tokenizer->setInputXml( options.doXMLin );
 	  tokenizer->setUttMarker( options.uttmark );
-	  tokenizer->setTextClass( options.textclass );
+	  tokenizer->setInputClass( options.inputclass );
+	  tokenizer->setOutputClass( options.outputclass );
+	  tokenizer->setTextRedundancy( options.textredundancy );
 	}
       }
 #pragma omp section
       {
 	if ( options.doLemma ){
-	  myMblem = new Mblem(theErrLog);
-	  lemStat = myMblem->init( configuration );
+	  try {
+	    myMblem = new Mblem(theErrLog);
+	    lemStat = myMblem->init( configuration );
+	  }
+	  catch ( const exception& e ){
+	    lemWhat = e.what();
+	    lemStat = false;
+	  }
 	}
       }
 #pragma omp section
       {
 	if ( options.doMorph ){
-	  myMbma = new Mbma(theErrLog);
-	  mbaStat = myMbma->init( configuration );
-	  if ( options.doDeepMorph )
-	    myMbma->setDeepMorph(true);
+	  try {
+	    myMbma = new Mbma(theErrLog);
+	    mbaStat = myMbma->init( configuration );
+	    if ( options.doDeepMorph )
+	      myMbma->setDeepMorph(true);
+	  }
+	  catch ( const exception& e ){
+	    mbaWhat = e.what();
+	    mbaStat = false;
+	  }
 	}
       }
 #pragma omp section
       {
-	myPoSTagger = new CGNTagger(theErrLog);
-	tagStat = myPoSTagger->init( configuration );
+	try {
+	  myCGNTagger = new CGNTagger( theErrLog );
+	  tagStat = myCGNTagger->init( configuration );
+	}
+	catch ( const exception& e ){
+	  tagWhat = e.what();
+	  tagStat = false;
+	}
       }
 #pragma omp section
       {
 	if ( options.doIOB ){
-	  myIOBTagger = new IOBTagger(theErrLog);
-	  iobStat = myIOBTagger->init( configuration );
+	  try {
+	    myIOBTagger = new IOBTagger( theErrLog );
+	    iobStat = myIOBTagger->init( configuration );
+	  }
+	  catch ( const exception& e ){
+	    iobWhat = e.what();
+	    iobStat = false;
+	  }
 	}
       }
 #pragma omp section
       {
 	if ( options.doNER ){
-	  myNERTagger = new NERTagger(theErrLog);
-	  nerStat = myNERTagger->init( configuration );
+	  try {
+	    myNERTagger = new NERTagger( theErrLog );
+	    nerStat = myNERTagger->init( configuration );
+	  }
+	  catch ( const exception& e ){
+	    nerWhat = e.what();
+	    nerStat = false;
+	  }
 	}
       }
 #pragma omp section
       {
 	if ( options.doMwu ){
-	  myMwu = new Mwu(theErrLog);
-	  mwuStat = myMwu->init( configuration );
-	  if ( mwuStat && options.doParse ){
-	    Timer initTimer;
-	    initTimer.start();
-	    myParser = new Parser(theErrLog);
-	    parStat = myParser->init( configuration );
-	    initTimer.stop();
-	    LOG << "init Parse took: " << initTimer << endl;
+	  try {
+	    myMwu = new Mwu(theErrLog);
+	    mwuStat = myMwu->init( configuration );
+	    if ( mwuStat && options.doParse ){
+	      TiCC::Timer initTimer;
+	      initTimer.start();
+	      try {
+		myParser = new Parser(theErrLog);
+		parStat = myParser->init( configuration );
+		initTimer.stop();
+		LOG << "init Parse took: " << initTimer << endl;
+	      }
+	      catch ( const exception& e ){
+		parWhat = e.what();
+		parStat = false;
+	      }
+	    }
+	  }
+	  catch ( const exception& e ){
+	    mwuWhat = e.what();
+	    mwuStat = false;
 	  }
 	}
       }
@@ -334,49 +457,49 @@ FrogAPI::FrogAPI( FrogOptions &opt,
 	     && mbaStat && mwuStat && parStat ) ){
       string out = "Initialization failed for: ";
       if ( !tokStat ){
-	out += "[tokenizer] ";
+	out += "[tokenizer] " + tokWhat;
       }
       if ( !tagStat ){
-	out += "[tagger] ";
+	out += "[tagger] " + tagWhat;
       }
       if ( !iobStat ){
-	out += "[IOB] ";
+	out += "[IOB] " + iobWhat;
       }
       if ( !nerStat ){
-	out += "[NER] ";
+	out += "[NER] " + nerWhat;
       }
       if ( !lemStat ){
-	out += "[lemmatizer] ";
+	out += "[lemmatizer] " + lemWhat;
       }
       if ( !mbaStat ){
-	out += "[morphology] ";
+	out += "[morphology] " + mbaWhat;
       }
       if ( !mwuStat ){
-	out += "[multiword unit] ";
+	out += "[multiword unit] " + mwuWhat;
       }
       if ( !parStat ){
-	out += "[parser] ";
+	out += "[parser] " + parWhat;
       }
       LOG << out << endl;
       throw runtime_error( "Frog init failed" );
     }
   }
-  LOG << "Initialization done." << endl;
+  LOG << TiCC::Timer::now() <<  " Initialization done." << endl;
 }
 
 FrogAPI::~FrogAPI() {
   delete myMbma;
   delete myMblem;
   delete myMwu;
-  delete myPoSTagger;
+  delete myCGNTagger;
   delete myIOBTagger;
   delete myNERTagger;
   delete myParser;
   delete tokenizer;
 }
 
-bool FrogAPI::TestSentence( Sentence* sent, TimerBlock& timers){
-  vector<Word*> swords;
+bool FrogAPI::TestSentence( folia::Sentence* sent, TimerBlock& timers){
+  vector<folia::Word*> swords;
   if ( options.doQuoteDetection ){
     swords = sent->wordParts();
   }
@@ -384,52 +507,18 @@ bool FrogAPI::TestSentence( Sentence* sent, TimerBlock& timers){
     swords = sent->words();
   }
   bool showParse = options.doParse;
-  bool all_well = true;
-  string exs;
   if ( !swords.empty() ) {
-#pragma omp parallel sections shared(all_well,exs)
-    {
-#pragma omp section
-      {
-	timers.tagTimer.start();
-	try {
-	  myPoSTagger->Classify( swords );
-	}
-	catch ( exception&e ){
-	  all_well = false;
-	  exs += string(e.what()) + " ";
-	}
-	timers.tagTimer.stop();
-      }
-#pragma omp section
-      {
-	if ( options.doIOB ){
-	  timers.iobTimer.start();
-	  try {
-	    myIOBTagger->Classify( swords );
-	  }
-	  catch ( exception&e ){
-	    all_well = false;
-	    exs += string(e.what()) + " ";
-	  }
-	  timers.iobTimer.stop();
-	}
-      }
-#pragma omp section
-      {
-	if ( options.doNER ){
-	  timers.nerTimer.start();
-	  try {
-	    myNERTagger->Classify( swords );
-	  }
-	  catch ( exception&e ){
-	    all_well = false;
-	    exs += string(e.what()) + " ";
-	  }
-	  timers.nerTimer.stop();
-	}
-      }
-    } // parallel sections
+    bool all_well = true;
+    string exs;
+    timers.tagTimer.start();
+    try {
+      myCGNTagger->Classify( swords );
+    }
+    catch ( exception&e ){
+      all_well = false;
+      exs += string(e.what()) + " ";
+    }
+    timers.tagTimer.stop();
     if ( !all_well ){
       throw runtime_error( exs );
     }
@@ -471,26 +560,65 @@ bool FrogAPI::TestSentence( Sentence* sent, TimerBlock& timers){
 	  }
 	}
       } // omp parallel sections
-    } //for int i = 0 to num_words
+    } //for all words
     if ( !all_well ){
       throw runtime_error( exs );
     }
-
-    if ( options.doMwu ){
-      if ( swords.size() > 0 ){
-	timers.mwuTimer.start();
-	myMwu->Classify( swords );
-	timers.mwuTimer.stop();
+#pragma omp parallel sections
+    {
+#pragma omp section
+      {
+	if ( options.doNER ){
+	  timers.nerTimer.start();
+	  if (options.debugFlag) {
+	    LOG << "Calling NER..." << endl;
+	  }
+	  try {
+	    myNERTagger->Classify( swords );
+	  }
+	  catch ( exception&e ){
+	    all_well = false;
+	    exs += string(e.what()) + " ";
+	  }
+	  timers.nerTimer.stop();
+	}
+      }
+#pragma omp section
+      {
+	if ( options.doIOB ){
+	  timers.iobTimer.start();
+	  try {
+	    myIOBTagger->Classify( swords );
+	  }
+	  catch ( exception&e ){
+	    all_well = false;
+	    exs += string(e.what()) + " ";
+	  }
+	  timers.iobTimer.stop();
+	}
+      }
+#pragma omp section
+      {
+	if ( options.doMwu ){
+	  if ( swords.size() > 0 ){
+	    timers.mwuTimer.start();
+	    myMwu->Classify( swords );
+	    timers.mwuTimer.stop();
+	  }
+	}
+	if ( options.doParse ){
+	  if ( options.maxParserTokens != 0
+	       && swords.size() > options.maxParserTokens ){
+	    showParse = false;
+	  }
+	  else {
+	    myParser->Parse( swords, timers );
+	  }
+	}
       }
     }
-    if ( options.doParse ){
-      if ( options.maxParserTokens != 0
-	   && swords.size() > options.maxParserTokens ){
-	showParse = false;
-      }
-      else {
-        myParser->Parse( swords, timers );
-      }
+    if ( !all_well ){
+      throw runtime_error( exs );
     }
   }
   return showParse;
@@ -498,15 +626,16 @@ bool FrogAPI::TestSentence( Sentence* sent, TimerBlock& timers){
 
 void FrogAPI::FrogServer( Sockets::ServerSocket &conn ){
   try {
-    while (true) {
+    while ( conn.isValid() ) {
       ostringstream outputstream;
       if ( options.doXMLin ){
         string result;
         string s;
         while ( conn.read(s) ){
-            result += s + "\n";
-            if ( s.empty() )
-                break;
+	  result += s + "\n";
+
+	  if ( s.empty() )
+	    break;
         }
         if ( result.size() < 50 ){
             // a FoLia doc must be at least a few 100 bytes
@@ -516,7 +645,7 @@ void FrogAPI::FrogServer( Sockets::ServerSocket &conn ){
         if ( options.debugFlag ){
 	  LOG << "received data [" << result << "]" << endl;
 	}
-        Document doc;
+	folia::Document doc;
         try {
 	  doc.readFromString( result );
         }
@@ -524,7 +653,7 @@ void FrogAPI::FrogServer( Sockets::ServerSocket &conn ){
 	  LOG << "FoLiaParsing failed:" << endl << e.what() << endl;
 	  throw;
         }
-        LOG << "Processing... " << endl;
+        LOG << "Processing XML... " << endl;
 	timers.reset();
 	timers.tokTimer.start();
 	tokenizer->tokenize( doc );
@@ -536,6 +665,7 @@ void FrogAPI::FrogServer( Sockets::ServerSocket &conn ){
 	else {
 	  showResults( outputstream, doc );
 	}
+	//        LOG << "Done Processing XML... " << endl;
       }
       else {
         string data = "";
@@ -556,11 +686,11 @@ void FrogAPI::FrogServer( Sockets::ServerSocket &conn ){
         if ( options.debugFlag ){
 	  LOG << "Received: [" << data << "]" << endl;
 	}
-        LOG << "Processing... " << endl;
+        LOG << TiCC::Timer::now() << " Processing... " << endl;
         istringstream inputstream(data,istringstream::in);
 	timers.reset();
 	timers.tokTimer.start();
-        Document *doc = tokenizer->tokenize( inputstream );
+	folia::Document *doc = tokenizer->tokenize( inputstream );
 	timers.tokTimer.stop();
         FrogDoc( *doc );
 	if ( options.doXMLout ){
@@ -570,6 +700,7 @@ void FrogAPI::FrogServer( Sockets::ServerSocket &conn ){
 	  showResults( outputstream, *doc );
 	}
 	delete doc;
+	//	LOG << "Done Processing... " << endl;
       }
       if (!conn.write( (outputstream.str()) ) || !(conn.write("READY\n"))  ){
 	if (options.debugFlag) {
@@ -580,9 +711,7 @@ void FrogAPI::FrogServer( Sockets::ServerSocket &conn ){
     }
   }
   catch ( std::exception& e ) {
-    if (options.debugFlag){
-      LOG << "connection lost: " << e.what() << endl;
-    }
+    LOG << "connection lost unexpected : " << e.what() << endl;
   }
   LOG << "Connection closed.\n";
 }
@@ -592,7 +721,6 @@ void FrogAPI::FrogStdin( bool prompt ) {
     cout << "frog>"; cout.flush();
   }
   string line;
-  string data;
   while ( getline( cin, line ) ){
     string data = line;
     if ( options.doSentencePerLine ){
@@ -630,7 +758,7 @@ void FrogAPI::FrogStdin( bool prompt ) {
       cout << "Processing... " << endl;
     }
     istringstream inputstream(data,istringstream::in);
-    Document *doc = tokenizer->tokenize( inputstream );
+    folia::Document *doc = tokenizer->tokenize( inputstream );
     FrogDoc( *doc, true );
     showResults( cout, *doc );
     delete doc;
@@ -659,7 +787,6 @@ void FrogAPI::FrogInteractive(){
       string data;
       char *input = readline( prompt );
       if ( !input ){
-	eof = true;
 	break;
       }
       line = input;
@@ -702,7 +829,7 @@ void FrogAPI::FrogInteractive(){
 	}
 	cout << "Processing... '" << data << "'" << endl;
 	istringstream inputstream(data,istringstream::in);
-	Document *doc = tokenizer->tokenize( inputstream );
+	folia::Document *doc = tokenizer->tokenize( inputstream );
 	FrogDoc( *doc, true );
 	showResults( cout, *doc );
 	delete doc;
@@ -713,49 +840,49 @@ void FrogAPI::FrogInteractive(){
 #endif
 }
 
-vector<Word*> FrogAPI::lookup( Word *word,
-			       const vector<Entity*>& entities ) const {
+vector<folia::Word*> FrogAPI::lookup( folia::Word *word,
+				      const vector<folia::Entity*>& entities ) const {
   for ( const auto& ent : entities ){
-    vector<Word*> vec = ent->select<Word>();
+    vector<folia::Word*> vec = ent->select<folia::Word>();
     if ( !vec.empty() ){
       if ( vec[0]->id() == word->id() ) {
 	return vec;
       }
     }
   }
-  vector<Word*> vec;
+  vector<folia::Word*> vec;
   vec.push_back( word ); // single unit
   return vec;
 }
 
-Dependency *FrogAPI::lookupDep( const Word *word,
-				const vector<Dependency*>&dependencies ) const{
+folia::Dependency *FrogAPI::lookupDep( const folia::Word *word,
+				       const vector<folia::Dependency*>&dependencies ) const{
   if (dependencies.size() == 0 ){
     return 0;
   }
   int dbFlag = 0;
   try {
-    dbFlag = stringTo<int>( configuration.lookUp( "debug", "parser" ) );
+    dbFlag = TiCC::stringTo<int>( configuration.lookUp( "debug", "parser" ) );
   }
   catch (exception & e) {
     dbFlag = 0;
   }
   if ( dbFlag ){
-    using TiCC::operator<<;
-    LOG << "\nDependency-lookup "<< word << " in " << dependencies << endl;
+    LOG << endl << "Dependency-lookup "<< word << " in " << dependencies << endl;
   }
   for ( const auto& dep : dependencies ){
     if ( dbFlag ){
       LOG << "Dependency try: " << dep << endl;
     }
     try {
-      vector<DependencyDependent*> dv = dep->select<DependencyDependent>();
+      vector<folia::DependencyDependent*> dv
+	= dep->select<folia::DependencyDependent>();
       if ( !dv.empty() ){
-	vector<Word*> wv = dv[0]->select<Word>();
+	vector<folia::Word*> wv = dv[0]->select<folia::Word>();
 	for ( const auto& w : wv ){
 	  if ( w == word ){
 	    if ( dbFlag ){
-	      LOG << "\nDependency found word " << w << endl;
+	      LOG << "Dependency found word " << w << endl;
 	    }
 	    return dep;
 	  }
@@ -764,28 +891,26 @@ Dependency *FrogAPI::lookupDep( const Word *word,
     }
     catch ( exception& e ){
       if (dbFlag > 0){
-	LOG << "get Dependency results failed: "
-			<< e.what() << endl;
+	LOG << "get Dependency results failed: " << e.what() << endl;
       }
     }
   }
   return 0;
 }
 
-string FrogAPI::lookupNEREntity( const vector<Word *>& mwus,
-				 const vector<Entity*>& entities ) const {
+string FrogAPI::lookupNEREntity( const vector<folia::Word *>& mwus,
+				 const vector<folia::Entity*>& entities ) const {
   string endresult;
   int dbFlag = 0;
   try{
-    dbFlag = stringTo<int>( configuration.lookUp( "debug", "NER" ) );
+    dbFlag = TiCC::stringTo<int>( configuration.lookUp( "debug", "NER" ) );
   }
   catch (exception & e) {
     dbFlag = 0;
   }
   for ( const auto& mwu : mwus ){
     if ( dbFlag ){
-      using TiCC::operator<<;
-      LOG << "\nNER: lookup "<< mwu << " in " << entities << endl;
+      LOG << endl << "NER: lookup "<< mwu << " in " << entities << endl;
     }
     string result;
     for ( const auto& entity :entities ){
@@ -793,7 +918,7 @@ string FrogAPI::lookupNEREntity( const vector<Word *>& mwus,
 	LOG << "NER: try: " << entity << endl;
       }
       try {
-	vector<Word*> wv = entity->select<Word>();
+	vector<folia::Word*> wv = entity->select<folia::Word>();
 	bool first = true;
 	for ( const auto& word : wv ){
 	  if ( word == mwu ){
@@ -801,10 +926,10 @@ string FrogAPI::lookupNEREntity( const vector<Word *>& mwus,
 	      LOG << "NER found word " << word << endl;
 	    }
 	    if ( first ){
-	      result += "B-" + uppercase(entity->cls());
+	      result += "B-" + TiCC::uppercase(entity->cls());
 	    }
 	    else {
-	      result += "I-" + uppercase(entity->cls());
+	      result += "I-" + TiCC::uppercase(entity->cls());
 	    }
 	    break;
 	  }
@@ -834,19 +959,18 @@ string FrogAPI::lookupNEREntity( const vector<Word *>& mwus,
 }
 
 
-string FrogAPI::lookupIOBChunk( const vector<Word *>& mwus,
-				const vector<Chunk*>& chunks ) const{
+string FrogAPI::lookupIOBChunk( const vector<folia::Word *>& mwus,
+				const vector<folia::Chunk*>& chunks ) const{
   string endresult;
   int dbFlag = 0;
   try {
-   dbFlag = stringTo<int>( configuration.lookUp( "debug", "IOB" ) );
+    dbFlag = TiCC::stringTo<int>( configuration.lookUp( "debug", "IOB" ) );
   }
   catch (exception & e) {
     dbFlag = 0;
   }
   for ( const auto& mwu : mwus ){
     if ( dbFlag ){
-      using TiCC::operator<<;
       LOG << "IOB lookup "<< mwu << " in " << chunks << endl;
     }
     string result;
@@ -855,7 +979,7 @@ string FrogAPI::lookupIOBChunk( const vector<Word *>& mwus,
 	LOG << "IOB try: " << chunk << endl;
       }
       try {
-	vector<Word*> wv = chunk->select<Word>();
+	vector<folia::Word*> wv = chunk->select<folia::Word>();
 	bool first = true;
 	for ( const auto& word : wv ){
 	  if ( word == mwu ){
@@ -897,18 +1021,16 @@ string FrogAPI::lookupIOBChunk( const vector<Word *>& mwus,
 
 vector<string> get_compound_analysis( folia::Word* word ){
   vector<string> result;
-  vector<MorphologyLayer*> layers
-    = word->annotations<MorphologyLayer>( Mbma::mbma_tagset );
+  vector<folia::MorphologyLayer*> layers
+    = word->annotations<folia::MorphologyLayer>( Mbma::mbma_tagset );
   for ( const auto& layer : layers ){
-    vector<Morpheme*> m =
-      layer->select<Morpheme>( Mbma::mbma_tagset, false );
+    vector<folia::Morpheme*> m =
+      layer->select<folia::Morpheme>( Mbma::mbma_tagset, false );
     if ( m.size() == 1 ) {
       // check for top layer compound
-      PosAnnotation *postag = 0;
       try {
-	postag = m[0]->annotation<PosAnnotation>( Mbma::clex_tagset );
-	//	cerr << "found a clex postag!" << endl;
-	result.push_back( postag->feat( "compound" ) ); // might be empty
+	folia::PosAnnotation *tag = m[0]->annotation<folia::PosAnnotation>( Mbma::clex_tagset );
+	result.push_back( tag->feat( "compound" ) ); // might be empty
       }
       catch (...){
 	result.push_back( "" ); // pad with empty strings
@@ -943,12 +1065,18 @@ string flatten( const string& s ){
 }
 
 vector<string> get_full_morph_analysis( folia::Word* w, bool flat ){
+  return get_full_morph_analysis( w, "current", flat );
+}
+
+vector<string> get_full_morph_analysis( folia::Word* w,
+					const string& cls,
+					bool flat ){
   vector<string> result;
-  vector<MorphologyLayer*> layers
-    = w->annotations<MorphologyLayer>( Mbma::mbma_tagset );
+  vector<folia::MorphologyLayer*> layers
+    = w->annotations<folia::MorphologyLayer>( Mbma::mbma_tagset );
   for ( const auto& layer : layers ){
-    vector<Morpheme*> m =
-      layer->select<Morpheme>( Mbma::mbma_tagset, false );
+    vector<folia::Morpheme*> m =
+      layer->select<folia::Morpheme>( Mbma::mbma_tagset, false );
     bool is_deep = false;
     if ( m.size() == 1 ) {
       // check for top layer from deep morph analysis
@@ -964,9 +1092,10 @@ vector<string> get_full_morph_analysis( folia::Word* w, bool flat ){
     if ( !is_deep ){
       // flat structure
       string morph;
-      vector<Morpheme*> m = layer->select<Morpheme>( Mbma::mbma_tagset );
+      vector<folia::Morpheme*> m
+	= layer->select<folia::Morpheme>( Mbma::mbma_tagset );
       for ( const auto& mor : m ){
-	string txt = UnicodeToUTF8( mor->text() );
+	string txt = TiCC::UnicodeToUTF8( mor->text( cls ) );
 	morph += "[" + txt + "]";
       }
       result.push_back( morph );
@@ -977,7 +1106,7 @@ vector<string> get_full_morph_analysis( folia::Word* w, bool flat ){
 
 void FrogAPI::displayMWU( ostream& os,
 			  size_t index,
-			  const vector<Word*>& mwu ) const {
+			  const vector<folia::Word*>& mwu ) const {
   string wrd;
   string pos;
   string lemma;
@@ -986,8 +1115,9 @@ void FrogAPI::displayMWU( ostream& os,
   double conf = 1;
   for ( const auto& word : mwu ){
     try {
-      wrd += word->str();
-      PosAnnotation *postag = word->annotation<PosAnnotation>( myPoSTagger->getTagset() );
+      wrd += word->str( options.outputclass );
+      folia::PosAnnotation *postag
+	= word->annotation<folia::PosAnnotation>( myCGNTagger->getTagset() );
       pos += postag->cls();
       if ( &word != &mwu.back() ){
 	wrd += "_";
@@ -1018,7 +1148,7 @@ void FrogAPI::displayMWU( ostream& os,
     if ( options.doMorph ){
       // also covers doDeepMorph
       try {
-	vector<string> morphs = get_full_morph_analysis( word );
+	vector<string> morphs = get_full_morph_analysis( word, options.outputclass );
 	for ( const auto& m : morphs ){
 	  morph += m;
 	  if ( &m != &morphs.back() ){
@@ -1073,54 +1203,54 @@ void FrogAPI::displayMWU( ostream& os,
 }
 
 ostream& FrogAPI::showResults( ostream& os,
-			       Document& doc ) const {
-  vector<Sentence*> sentences = doc.sentences();
+			       folia::Document& doc ) const {
+  vector<folia::Sentence*> sentences = doc.sentences();
   for ( auto const& sentence : sentences ){
-    vector<Word*> words = sentence->words();
-    vector<Entity*> mwu_entities;
+    vector<folia::Word*> words = sentence->words();
+    vector<folia::Entity*> mwu_entities;
     if (myMwu){
-      mwu_entities = sentence->select<Entity>( myMwu->getTagset() );
+      mwu_entities = sentence->select<folia::Entity>( myMwu->getTagset() );
     }
-    vector<Dependency*> dependencies;
+    vector<folia::Dependency*> dependencies;
     if (myParser){
-      dependencies = sentence->select<Dependency>( myParser->getTagset() );
+      dependencies = sentence->select<folia::Dependency>( myParser->getTagset() );
     }
-    vector<Chunk*> iob_chunking;
+    vector<folia::Chunk*> iob_chunking;
     if ( myIOBTagger ){
-      iob_chunking = sentence->select<Chunk>( myIOBTagger->getTagset() );
+      iob_chunking = sentence->select<folia::Chunk>( myIOBTagger->getTagset() );
     }
-    vector<Entity*> ner_entities;
+    vector<folia::Entity*> ner_entities;
     if (myNERTagger){
-      ner_entities =  sentence->select<Entity>( myNERTagger->getTagset() );
+      ner_entities =  sentence->select<folia::Entity>( myNERTagger->getTagset() );
     }
-    static set<ElementType> excludeSet;
-    vector<Sentence*> parts = sentence->select<Sentence>( excludeSet );
+    static set<folia::ElementType> excludeSet;
+    vector<folia::Sentence*> parts = sentence->select<folia::Sentence>( excludeSet );
     if ( !options.doQuoteDetection ){
       assert( parts.size() == 0 );
     }
     for ( auto const& part : parts ){
-      vector<Entity*> ents;
+      vector<folia::Entity*> ents;
       if (myMwu){
-	ents = part->select<Entity>( myMwu->getTagset() );
+	ents = part->select<folia::Entity>( myMwu->getTagset() );
       }
       mwu_entities.insert( mwu_entities.end(), ents.begin(), ents.end() );
-      vector<Dependency*> deps = part->select<Dependency>();
+      vector<folia::Dependency*> deps = part->select<folia::Dependency>();
       dependencies.insert( dependencies.end(), deps.begin(), deps.end() );
-      vector<Chunk*> chunks = part->select<Chunk>();
+      vector<folia::Chunk*> chunks = part->select<folia::Chunk>();
       iob_chunking.insert( iob_chunking.end(), chunks.begin(), chunks.end() );
-      vector<Entity*> ners ;
+      vector<folia::Entity*> ners ;
       if (myNERTagger) {
-	ners = part->select<Entity>( myNERTagger->getTagset() );
+	ners = part->select<folia::Entity>( myNERTagger->getTagset() );
       }
       ner_entities.insert( ner_entities.end(), ners.begin(), ners.end() );
     }
 
     size_t index = 1;
-    map<FoliaElement*, int> enumeration;
-    vector<vector<Word*> > mwus;
+    unordered_map<folia::FoliaElement*, int> enumeration;
+    vector<vector<folia::Word*> > mwus;
     for ( size_t i=0; i < words.size(); ++i ){
-      Word *word = words[i];
-      vector<Word*> mwu = lookup( word, mwu_entities );
+      folia::Word *word = words[i];
+      vector<folia::Word*> mwu = lookup( word, mwu_entities );
       for ( size_t j=0; j < mwu.size(); ++j ){
 	enumeration[mwu[j]] = index;
       }
@@ -1132,7 +1262,6 @@ ostream& FrogAPI::showResults( ostream& os,
     for ( const auto& mwu : mwus ){
       displayMWU( os, ++index, mwu );
       if ( options.doNER ){
-	string cls;
 	string s = lookupNEREntity( mwu, ner_entities );
 	os << "\t" << s;
       }
@@ -1140,7 +1269,6 @@ ostream& FrogAPI::showResults( ostream& os,
 	os << "\t\t";
       }
       if ( options.doIOB ){
-	string cls;
 	string s = lookupIOBChunk( mwu, iob_chunking);
 	os << "\t" << s;
       }
@@ -1148,14 +1276,13 @@ ostream& FrogAPI::showResults( ostream& os,
 	os << "\t\t";
       }
       if ( options.doParse ){
-	string cls;
-	Dependency *dep = lookupDep( mwu[0], dependencies);
+	folia::Dependency *dep = lookupDep( mwu[0], dependencies);
 	if ( dep ){
-	  vector<Headspan*> w = dep->select<Headspan>();
+	  vector<folia::Headspan*> w = dep->select<folia::Headspan>();
 	  size_t num;
-	  if ( w[0]->index(0)->isinstance( PlaceHolder_t ) ){
+	  if ( w[0]->index(0)->isinstance( folia::PlaceHolder_t ) ){
 	    string indexS = w[0]->index(0)->str();
-	    FoliaElement *pnt = w[0]->index(0)->doc()->index(indexS);
+	    folia::FoliaElement *pnt = w[0]->index(0)->doc()->index(indexS);
 	    num = enumeration.find(pnt->index(0))->second;
 	  }
 	  else {
@@ -1180,10 +1307,15 @@ ostream& FrogAPI::showResults( ostream& os,
 }
 
 string FrogAPI::Frogtostring( const string& s ){
-  Document *doc = tokenizer->tokenizestring( s );
+  folia::Document *doc = tokenizer->tokenizestring( s );
   stringstream ss;
   FrogDoc( *doc, true );
-  showResults( ss, *doc );
+  if ( options.doXMLout ){
+    doc->save( ss, options.doKanon );
+  }
+  else {
+    showResults( ss, *doc );
+  }
   delete doc;
   return ss.str();
 }
@@ -1194,13 +1326,13 @@ string FrogAPI::Frogtostringfromfile( const string& name ){
   return ss.str();
 }
 
-void FrogAPI::FrogDoc( Document& doc,
+void FrogAPI::FrogDoc( folia::Document& doc,
 		       bool hidetimers ){
   timers.frogTimer.start();
   // first we make sure that the doc will accept our annotations, by
   // declaring them in the doc
-  if (myPoSTagger){
-    myPoSTagger->addDeclaration( doc );
+  if (myCGNTagger){
+    myCGNTagger->addDeclaration( doc );
   }
   if ( options.doLemma && myMblem ) {
     myMblem->addDeclaration( doc );
@@ -1223,8 +1355,7 @@ void FrogAPI::FrogDoc( Document& doc,
   if ( options.debugFlag > 5 ){
     LOG << "Testing document :" << doc << endl;
   }
-
-  vector<Sentence*> sentences;
+  vector<folia::Sentence*> sentences;
   if ( options.doQuoteDetection ){
     sentences = doc.sentenceParts();
   }
@@ -1233,10 +1364,7 @@ void FrogAPI::FrogDoc( Document& doc,
   }
   size_t numS = sentences.size();
   if ( numS > 0 ) { //process sentences
-    if  (options.debugFlag > 0) {
-      LOG << "found " << numS
-		      << " sentence(s) in document." << endl;
-    }
+    LOG << TiCC::Timer::now() << " process " << numS << " sentences" << endl;
     for ( size_t i = 0; i < numS; ++i ) {
       //NOTE- full sentences are passed (which may span multiple lines) (MvG)
       string lan = sentences[i]->language();
@@ -1255,8 +1383,11 @@ void FrogAPI::FrogDoc( Document& doc,
       if ( options.doParse && !showParse ){
 	LOG << "WARNING!" << endl;
 	LOG << "Sentence " << i+1
-			<< " isn't parsed because it contains more tokens then set with the --max-parser-tokens="
-			<< options.maxParserTokens << " option." << endl;
+	    << " isn't parsed because it contains more tokens then set with the --max-parser-tokens="
+	    << options.maxParserTokens << " option." << endl;
+      }
+      else {
+	LOG << TiCC::Timer::now() << " done with sentence[" << i+1 << "]" << endl;
       }
     }
   }
@@ -1298,34 +1429,71 @@ void FrogAPI::FrogDoc( Document& doc,
   return;
 }
 
+string filter_non_NC( const string& filename ){
+  string result;
+  bool at_start = true;
+  for ( size_t i=0; i < filename.length(); ++i ){
+    if ( at_start ){
+      if ( isdigit( filename[i] ) ){
+	continue; //ditch numerics!
+      }
+      else {
+	at_start = false;
+      }
+    }
+    if ( filename[i] == ':' ){
+      if ( at_start ){
+	result += "C";
+      }
+      else {
+	result += "-";
+      }
+    }
+    else {
+      result += filename[i];
+    }
+  }
+  if ( result.empty() ){
+    // ouch, only numbers?
+    return filter_non_NC( "N" + filename );
+  }
+  return result;
+}
+
 void FrogAPI::FrogFile( const string& infilename,
-			ostream &os,
+			ostream& os,
 			const string& xmlOutF ) {
   // stuff the whole input into one FoLiA document.
   // This is not a good idea on the long term, I think (agreed [proycon] )
   string xmlOutFile = xmlOutF;
-  if ( options.doXMLin && !xmlOutFile.empty() ){
-    if ( match_back( infilename, ".gz" ) ){
-      if ( !match_back( xmlOutFile, ".gz" ) )
+  bool xml_in = options.doXMLin;
+  if ( TiCC::match_back( infilename, ".xml.gz" )
+       || TiCC::match_back( infilename, ".xml.bz2" )
+       || TiCC::match_back( infilename, ".xml" ) ){
+    xml_in = true;
+  }
+  if ( xml_in && !xmlOutFile.empty() ){
+    if ( TiCC::match_back( infilename, ".gz" ) ){
+      if ( !TiCC::match_back( xmlOutFile, ".gz" ) )
 	xmlOutFile += ".gz";
     }
-    else if ( match_back( infilename, ".bz2" ) ){
-      if ( !match_back( xmlOutFile, ".bz2" ) )
+    else if ( TiCC::match_back( infilename, ".bz2" ) ){
+      if ( !TiCC::match_back( xmlOutFile, ".bz2" ) )
 	xmlOutFile += ".bz2";
     }
   }
-  if ( options.doXMLin ){
-    Document doc;
+  if ( xml_in ){
+    timers.reset();
+    timers.tokTimer.start();
+    folia::Document doc;
     try {
       doc.readFromFile( infilename );
     }
     catch ( exception &e ){
       LOG << "retrieving FoLiA from '" << infilename << "' failed with exception:" << endl;
-      cerr << e.what() << endl;
-      return;
+      LOG << e.what() << endl;
+      throw ( runtime_error( "read failed" ) );
     }
-    timers.reset();
-    timers.tokTimer.start();
     tokenizer->tokenize( doc );
     timers.tokTimer.stop();
     FrogDoc( doc );
@@ -1333,20 +1501,28 @@ void FrogAPI::FrogFile( const string& infilename,
       doc.save( xmlOutFile, options.doKanon );
       LOG << "resulting FoLiA doc saved in " << xmlOutFile << endl;
     }
-    showResults( os, doc );
+    if ( !options.noStdOut ){
+      showResults( os, doc );
+    }
   }
   else {
     ifstream IN( infilename );
     timers.reset();
     timers.tokTimer.start();
-    Document *doc = tokenizer->tokenize( IN );
+    if ( options.docid == "untitled" ){
+      string id = filter_non_NC( TiCC::basename(infilename) );
+      tokenizer->setDocID( id );
+    }
+    folia::Document *doc = tokenizer->tokenize( IN );
     timers.tokTimer.stop();
     FrogDoc( *doc );
     if ( !xmlOutFile.empty() ){
       doc->save( xmlOutFile, options.doKanon );
       LOG << "resulting FoLiA doc saved in " << xmlOutFile << endl;
     }
-    showResults( os, *doc );
+    if ( !options.noStdOut ){
+      showResults( os, *doc );
+    }
     delete doc;
   }
 }
