@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -29,23 +29,24 @@
 
 */
 
+#include "frog/Parser.h"
+
 #include <sys/types.h>
 #include <unistd.h>
 #include <cstdlib>
 #include <string>
 #include <iostream>
 #include <fstream>
+#include <algorithm>
 
 #include "config.h"
 #include "ticcutils/Configuration.h"
+#include "ticcutils/PrettyPrint.h"
 #include "timbl/TimblAPI.h"
-#include "ucto/unicode.h"
-#include "frog/Frog.h"
-#include "frog/Parser.h"
+#include "frog/Frog-util.h"
 #include "frog/csidp.h"
 
 using namespace std;
-using namespace folia;
 
 using TiCC::operator<<;
 
@@ -62,7 +63,7 @@ struct parseData {
   vector<string> words;
   vector<string> heads;
   vector<string> mods;
-  vector<vector<Word*> > mwus;
+  vector<vector<folia::Word*> > mwus;
 };
 
 ostream& operator<<( ostream& os, const parseData& pd ){
@@ -92,7 +93,16 @@ bool Parser::init( const TiCC::Configuration& configuration ){
   bool problem = false;
   LOG << "initiating parser ... " << endl;
   string cDir = configuration.configDir();
-  string val = configuration.lookUp( "version", "parser" );
+  string val = configuration.lookUp( "debug", "parser" );
+  if ( !val.empty() ){
+    int level;
+    if ( TiCC::stringTo<int>( val, level ) ){
+      if ( level > 5 ){
+	parseLog->setlevel( LogLevel::LogDebug );
+      }
+    }
+  }
+  val = configuration.lookUp( "version", "parser" );
   if ( val.empty() ){
     version = "1.0";
   }
@@ -125,7 +135,7 @@ bool Parser::init( const TiCC::Configuration& configuration ){
     charFile = configuration.lookUp( "char_filter_file" );
   if ( !charFile.empty() ){
     charFile = prefix( configuration.configDir(), charFile );
-    filter = new Tokenizer::UnicodeFilter();
+    filter = new TiCC::UniFilter();
     filter->fill( charFile );
   }
   val = configuration.lookUp( "maxDepSpan", "parser" );
@@ -181,6 +191,13 @@ bool Parser::init( const TiCC::Configuration& configuration ){
     return false;
   }
 
+  string cls = configuration.lookUp( "outputclass" );
+  if ( !cls.empty() ){
+    textclass = cls;
+  }
+  else {
+    textclass = "current";
+  }
   bool happy = true;
   pairs = new Timbl::TimblAPI( pairsOptions );
   if ( pairs->Valid() ){
@@ -227,11 +244,11 @@ Parser::~Parser(){
   delete filter;
 }
 
-static vector<Word *> lookup( Word *word,
-			      const vector<Entity*>& entities ){
-  vector<Word*> vec;
+static vector<folia::Word *> lookup( folia::Word *word,
+				     const vector<folia::Entity*>& entities ){
+  vector<folia::Word*> vec;
   for ( const auto& ent : entities ){
-    vec = ent->select<Word>();
+    vec = ent->select<folia::Word>();
     if ( !vec.empty() ){
       if ( vec[0]->id() == word->id() ) {
 	// cerr << "found " << vec << endl;
@@ -515,13 +532,13 @@ vector<string> Parser::createDirInstances( const parseData& pd ){
   }
   else {
     for ( size_t i=0 ; i < words.size(); ++i ){
-      string word_0, word_1, word_2;
-      string tag_0, tag_1, tag_2;
-      string mod_0, mod_1, mod_2;
+      string word_1, word_2;
+      string tag_1, tag_2;
+      string mod_1;
       if ( i == 0 ){
 	word_2 = "__";
 	tag_2 = "__";
-	mod_2 = "__";
+	//	mod_2 = "__";
 	word_1 = "__";
 	tag_1 = "__";
 	mod_1 = "__";
@@ -529,7 +546,7 @@ vector<string> Parser::createDirInstances( const parseData& pd ){
       else if ( i == 1 ){
 	word_2 = "__";
 	tag_2 = "__";
-	mod_2 = "__";
+	//	mod_2 = "__";
 	word_1 = words[i-1];
 	tag_1 = heads[i-1];
 	mod_1 = mods[i-1];
@@ -537,7 +554,7 @@ vector<string> Parser::createDirInstances( const parseData& pd ){
       else {
 	word_2 = words[i-2];
 	tag_2 = heads[i-2];
-	mod_2 = mods[i-2];
+	//	mod_2 = mods[i-2];
 	word_1 = words[i-1];
 	tag_1 = heads[i-1];
 	mod_1 = mods[i-1];
@@ -547,14 +564,14 @@ vector<string> Parser::createDirInstances( const parseData& pd ){
       string tag0 = heads[i];
       string tag1, tag2;
       string mod0 = mods[i];
-      string mod1, mod2;
+      string mod1;
       if ( i < words.size() - 2 ){
 	word1 = words[i+1];
 	tag1 = heads[i+1];
 	mod1 = mods[i+1];
 	word2 = words[i+2];
 	tag2 = heads[i+2];
-	mod2 = mods[i+2];
+	//	mod2 = mods[i+2];
       }
       else if ( i == words.size() - 2 ){
 	word1 = words[i+1];
@@ -562,7 +579,7 @@ vector<string> Parser::createDirInstances( const parseData& pd ){
 	mod1 = mods[i+1];
 	word2 = "__";
 	tag2 = "__";
-	mod2 = "__";
+	//	mod2 = "__";
       }
       else {
 	word1 = "__";
@@ -570,7 +587,7 @@ vector<string> Parser::createDirInstances( const parseData& pd ){
 	mod1 = "__";
 	word2 = "__";
 	tag2 = "__";
-	mod2 = "__";
+	//	mod2 = "__";
       }
       string inst = word_2
 	+ " " + word_1
@@ -715,62 +732,48 @@ vector<string> Parser::createRelInstances( const parseData& pd ){
   }
   else {
     for ( size_t i=0 ; i < words.size(); ++i ){
-      string word_0, word_1, word_2;
-      string tag_0, tag_1, tag_2;
-      string mod_0, mod_1, mod_2;
+      string word_1, word_2;
+      string tag_1, tag_2;
       if ( i == 0 ){
 	word_2 = "__";
 	tag_2 = "__";
-	mod_2 = "__";
 	word_1 = "__";
 	tag_1 = "__";
-	mod_1 = "__";
       }
       else if ( i == 1 ){
 	word_2 = "__";
 	tag_2 = "__";
-	mod_2 = "__";
 	word_1 = words[i-1];
 	tag_1 = heads[i-1];
-	mod_1 = mods[i-1];
       }
       else {
 	word_2 = words[i-2];
 	tag_2 = heads[i-2];
-	mod_2 = mods[i-2];
 	word_1 = words[i-1];
 	tag_1 = heads[i-1];
-	mod_1 = mods[i-1];
       }
       string word0 = words[i];
       string word1, word2;
       string tag0 = heads[i];
       string tag1, tag2;
       string mod0 = mods[i];
-      string mod1, mod2;
       if ( i < words.size() - 2 ){
 	word1 = words[i+1];
 	tag1 = heads[i+1];
-	mod1 = mods[i+1];
 	word2 = words[i+2];
 	tag2 = heads[i+2];
-	mod2 = mods[i+2];
       }
       else if ( i == words.size() - 2 ){
 	word1 = words[i+1];
 	tag1 = heads[i+1];
-	mod1 = mods[i+1];
 	word2 = "__";
 	tag2 = "__";
-	mod2 = "__";
       }
       else {
 	word1 = "__";
 	tag1 = "__";
-	mod1 = "__";
 	word2 = "__";
 	tag2 = "__";
-	mod2 = "__";
       }
       //
       string inst = word_2
@@ -796,44 +799,44 @@ vector<string> Parser::createRelInstances( const parseData& pd ){
 }
 
 
-void Parser::addDeclaration( Document& doc ) const {
-#pragma omp critical(foliaupdate)
-  {
-    doc.declare( AnnotationType::DEPENDENCY, dep_tagset,
-		 "annotator='frog-depparse-" + version
-		 + "', annotatortype='auto'");
-  }
+void Parser::addDeclaration( folia::Document& doc ) const {
+  doc.declare( folia::AnnotationType::DEPENDENCY, dep_tagset,
+	       "annotator='frog-depparse-" + version
+	       + "', annotatortype='auto'");
 }
 
-parseData Parser::prepareParse( const vector<Word *>& fwords ){
+parseData Parser::prepareParse( const vector<folia::Word *>& fwords ){
   parseData pd;
-  Sentence *sent = 0;
-  vector<Entity*> entities;
-#pragma omp critical(foliaupdate)
+  folia::Sentence *sent = 0;
+  vector<folia::Entity*> entities;
+#pragma omp critical (foliaupdate)
   {
     sent = fwords[0]->sentence();
-    entities = sent->select<Entity>(MWU_tagset);
+    entities = sent->select<folia::Entity>(MWU_tagset);
   }
   for ( size_t i=0; i < fwords.size(); ++i ){
-    Word *word = fwords[i];
-    vector<Word*> mwuv = lookup( word, entities );
+    folia::Word *word = fwords[i];
+    vector<folia::Word*> mwuv = lookup( word, entities );
     if ( !mwuv.empty() ){
       string multi_word;
       string head;
       string mod;
       for ( const auto& mwu : mwuv ){
-	UnicodeString tmp;
-#pragma omp critical(foliaupdate)
+	icu::UnicodeString tmp;
+#pragma omp critical (foliaupdate)
 	{
-	  tmp = mwu->text();
+	  tmp = mwu->text( textclass );
 	}
 	if ( filter )
 	  tmp = filter->filter( tmp );
-	string ms = UnicodeToUTF8( tmp );
+	string ms = TiCC::UnicodeToUTF8( tmp );
+	// the word may contain spaces, remove them all!
+	ms.erase(remove_if(ms.begin(), ms.end(), ::isspace), ms.end());
 	multi_word += ms;
-	PosAnnotation *postag = mwu->annotation<PosAnnotation>( POS_tagset );
+	folia::PosAnnotation *postag
+	  = mwu->annotation<folia::PosAnnotation>( POS_tagset );
 	head += postag->feat("head");
-	vector<Feature*> feats = postag->select<Feature>();
+	vector<folia::Feature*> feats = postag->select<folia::Feature>();
 	for ( const auto& feat : feats ){
 	  mod += feat->cls();
 	  if ( &feat != &feats.back() ){
@@ -853,20 +856,23 @@ parseData Parser::prepareParse( const vector<Word *>& fwords ){
       i += mwuv.size()-1;
     }
     else {
-      UnicodeString tmp;
-#pragma omp critical(foliaupdate)
+      icu::UnicodeString tmp;
+#pragma omp critical (foliaupdate)
       {
-	tmp = word->text();
+	tmp = word->text( textclass );
       }
       if ( filter )
 	tmp = filter->filter( tmp );
-      string ms = UnicodeToUTF8( tmp );
+      string ms = TiCC::UnicodeToUTF8( tmp );
+      // the word may contain spaces, remove them all!
+      ms.erase(remove_if(ms.begin(), ms.end(), ::isspace), ms.end());
       pd.words.push_back( ms );
-      PosAnnotation *postag = word->annotation<PosAnnotation>( POS_tagset );
+      folia::PosAnnotation *postag
+	= word->annotation<folia::PosAnnotation>( POS_tagset );
       string head = postag->feat("head");
       pd.heads.push_back( head );
       string mod;
-      vector<Feature*> feats = postag->select<Feature>();
+      vector<folia::Feature*> feats = postag->select<folia::Feature>();
       if ( feats.empty() ){
 	mod = "__";
       }
@@ -879,7 +885,7 @@ parseData Parser::prepareParse( const vector<Word *>& fwords ){
 	}
       }
       pd.mods.push_back( mod );
-      vector<Word*> vec;
+      vector<folia::Word*> vec;
       vec.push_back(word);
       pd.mwus.push_back( vec );
     }
@@ -887,36 +893,45 @@ parseData Parser::prepareParse( const vector<Word *>& fwords ){
   return pd;
 }
 
-void appendResult( const vector<Word *>& words,
+void appendResult( const vector<folia::Word *>& words,
 		   parseData& pd,
 		   const string& tagset,
+		   const string& textclass,
 		   const vector<int>& nums,
 		   const vector<string>& roles ){
-  Sentence *sent = words[0]->sentence();
-  KWargs args;
+  folia::Sentence *sent = 0;
+#pragma omp critical (foliaupdate)
+  {
+    sent = words[0]->sentence();
+  }
+  folia::DependenciesLayer *dl = 0;
+  folia::KWargs args;
   args["generate_id"] = sent->id();
   args["set"] = tagset;
-  DependenciesLayer *dl = new DependenciesLayer( args, sent->doc() );
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
   {
+    dl = new folia::DependenciesLayer( args, sent->doc() );
     sent->append( dl );
   }
   for ( size_t i=0; i < nums.size(); ++i ){
     if ( nums[i] != 0 ){
-      KWargs args;
+      folia::KWargs args;
       args["generate_id"] = dl->id();
       args["class"] = roles[i];
       args["set"] = tagset;
-#pragma omp critical(foliaupdate)
+      if ( textclass != "current" ){
+	args["textclass"] = textclass;
+      }
+#pragma omp critical (foliaupdate)
       {
-	Dependency *d = new Dependency( args, sent->doc() );
+	folia::Dependency *d = new folia::Dependency( args, sent->doc() );
 	dl->append( d );
-	Headspan *dh = new Headspan();
+	folia::Headspan *dh = new folia::Headspan();
 	for ( const auto& wrd : pd.mwus[nums[i]-1] ){
 	  dh->append( wrd );
 	}
 	d->append( dh );
-	DependencyDependent *dd = new DependencyDependent();
+	folia::DependencyDependent *dd = new folia::DependencyDependent();
 	for ( const auto& it : pd.mwus[i] ){
 	  dd->append( it );
 	}
@@ -926,9 +941,10 @@ void appendResult( const vector<Word *>& words,
   }
 }
 
-void appendParseResult( const vector<Word *>& words,
+void appendParseResult( const vector<folia::Word *>& words,
 			parseData& pd,
 			const string& tagset,
+			const string& textclass,
 			const vector<parsrel>& res ){
   vector<int> nums;
   vector<string> roles;
@@ -936,7 +952,7 @@ void appendParseResult( const vector<Word *>& words,
     nums.push_back( it.head );
     roles.push_back( it.deprel );
   }
-  appendResult( words, pd, tagset, nums, roles );
+  appendResult( words, pd, tagset, textclass, nums, roles );
 }
 
 void timbl( Timbl::TimblAPI* tim,
@@ -950,7 +966,7 @@ void timbl( Timbl::TimblAPI* tim,
   }
 }
 
-void Parser::Parse( const vector<Word*>& words,
+void Parser::Parse( const vector<folia::Word*>& words,
 		    TimerBlock& timers ){
   timers.parseTimer.start();
   if ( !isInit ){
@@ -997,8 +1013,9 @@ void Parser::Parse( const vector<Word*>& words,
 			       r_results,
 			       d_results,
 			       pd.words.size(),
-			       maxDepSpan );
+			       maxDepSpan,
+			       parseLog );
   timers.csiTimer.stop();
-  appendParseResult( words, pd, dep_tagset, res );
+  appendParseResult( words, pd, dep_tagset, textclass, res );
   timers.parseTimer.stop();
 }
