@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -29,17 +29,13 @@
 
 */
 
-#include "mbt/MbtAPI.h"
-#include "ucto/unicode.h"
-#include "frog/Frog.h"
 #include "frog/cgn_tagger_mod.h"
+#include "frog/Frog-util.h"
 
 using namespace std;
-using namespace folia;
-using namespace TiCC;
 using namespace Tagger;
 
-#define LOG *Log(tag_log)
+#define LOG *TiCC::Log(tag_log)
 
 void CGNTagger::fillSubSetTable(){
   // should become a config file!
@@ -167,24 +163,32 @@ void CGNTagger::fillSubSetTable(){
 }
 
 
-bool CGNTagger::init( const Configuration& config ){
-  if ( debug ){
+bool CGNTagger::init( const TiCC::Configuration& config ){
+  if (  debug ){
     LOG << "INIT CGN Tagger." << endl;
   }
-  if ( POSTagger::init( config ) ){
+  if ( BaseTagger::init( config ) ){
     fillSubSetTable();
     if ( debug ){
-      LOG << "DONE CGN Tagger." << endl;
+      LOG << "DONE Init CGN Tagger." << endl;
     }
     return true;
   }
   return false;
 }
 
+void CGNTagger::addDeclaration( folia::Document& doc ) const {
+  doc.declare( folia::AnnotationType::POS,
+	       tagset,
+	       "annotator='frog-mbpos-" + version
+	       + "', annotatortype='auto', datetime='" + getTime() + "'");
+}
+
 string CGNTagger::getSubSet( const string& val, const string& head ){
   auto it = cgnSubSets.find( val );
-  if ( it == cgnSubSets.end() )
+  if ( it == cgnSubSets.end() ){
     throw folia::ValueError( "unknown cgn subset for class: '" + val + "'" );
+  }
   string result;
   while ( it != cgnSubSets.upper_bound(val) ){
     result = it->second;
@@ -208,30 +212,71 @@ string CGNTagger::getSubSet( const string& val, const string& head ){
 			   "' whithin the constraints for '" + head + "'" );
 }
 
-void CGNTagger::post_process( const vector<Word *>& words ){
+void CGNTagger::addTag( folia::Word *word,
+			const string& inputTag,
+			double confidence ){
+  string pos_tag = inputTag;
+  string ucto_class = word->cls();
+  if ( debug ){
+    LOG << "lookup ucto class= " << ucto_class << endl;
+  }
+  auto const tt = token_tag_map.find( ucto_class );
+  if ( tt != token_tag_map.end() ){
+    if ( debug ){
+      LOG << "found translation ucto class= " << ucto_class
+	  << " to POS-Tag=" << tt->second << endl;
+    }
+    pos_tag = tt->second;
+    confidence = 1.0;
+  }
+  folia::KWargs args;
+  args["set"]  = tagset;
+  args["class"]  = pos_tag;
+  args["confidence"]= TiCC::toString(confidence);
+  if ( textclass != "current" ){
+    args["textclass"] = textclass;
+  }
+#pragma omp critical (foliaupdate)
+  {
+    word->addPosAnnotation( args );
+  }
+}
+
+
+void CGNTagger::post_process( const vector<folia::Word *>& words ){
+  for ( size_t i=0; i < _tag_result.size(); ++i ){
+    addTag( words[i],
+	    _tag_result[i].assignedTag(),
+	    _tag_result[i].confidence() );
+  }
   for ( auto const& word : words ){
-    PosAnnotation *postag = word->annotation<PosAnnotation>( );
+    folia::PosAnnotation *postag = 0;
+#pragma omp critical (foliaupdate)
+    {
+      postag = word->annotation<folia::PosAnnotation>( );
+    }
     string cls = postag->cls();
-    vector<string> parts;
-    TiCC::split_at_first_of( cls, parts, "()" );
+    vector<string> parts = TiCC::split_at_first_of( cls, "()" );
     string head = parts[0];
-    KWargs args;
+    folia::KWargs args;
     args["class"]  = head;
     args["set"]    = tagset;
     folia::Feature *feat = new folia::HeadFeature( args );
-    postag->append( feat );
-    if ( head == "SPEC" ){
-      postag->confidence(1.0);
+#pragma omp critical (foliaupdate)
+    {
+      postag->append( feat );
+      if ( head == "SPEC" ){
+	postag->confidence(1.0);
+      }
     }
     if ( parts.size() > 1 ){
-      vector<string> tagParts;
-      TiCC::split_at( parts[1], tagParts, "," );
+      vector<string> tagParts = TiCC::split_at( parts[1], "," );
       for ( auto const& part : tagParts ){
-	KWargs args;
+	folia::KWargs args;
 	args["set"]    = tagset;
 	args["subset"] = getSubSet( part, head );
 	args["class"]  = part;
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
 	{
 	  folia::Feature *feat = new folia::Feature( args );
 	  postag->append( feat );
@@ -240,11 +285,3 @@ void CGNTagger::post_process( const vector<Word *>& words ){
     }
   }
 }
-
-void CGNTagger::Classify( const vector<Word*>& swords ){
-  POSTagger::Classify( swords );
-  if ( debug ){
-    LOG << "POS Classify done:" << endl;
-  }
-  post_process( swords );
-}
