@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -29,24 +29,21 @@
 
 */
 
-#include <cstdlib>
+#include "frog/mblem_mod.h"
+
 #include <string>
 #include <iostream>
 #include <fstream>
 #include "timbl/TimblAPI.h"
-#include "ucto/unicode.h"
 #include "ticcutils/LogStream.h"
 #include "ticcutils/Configuration.h"
-#include "frog/Frog.h"
-#include "frog/mblem_mod.h"
+#include "frog/Frog-util.h"
 
 using namespace std;
-using namespace TiCC;
-using namespace folia;
 
-#define LOG *Log(mblemLog)
+#define LOG *TiCC::Log(mblemLog)
 
-Mblem::Mblem( LogStream *logstream ):
+Mblem::Mblem( TiCC::LogStream *logstream ):
   myLex(0),
   punctuation( "?...,:;\\'`(){}[]%#+-_=/!" ),
   history(20),
@@ -54,7 +51,7 @@ Mblem::Mblem( LogStream *logstream ):
   keep_case( false ),
   filter(0)
 {
-  mblemLog = new LogStream( logstream, "mblem" );
+  mblemLog = new TiCC::LogStream( logstream, "mblem" );
 }
 
 bool Mblem::fill_ts_map( const string& file ){
@@ -67,8 +64,8 @@ bool Mblem::fill_ts_map( const string& file ){
   while ( getline( is, line ) ){
     if ( line.empty() || line[0] == '#' )
       continue;
-    vector<string> parts;
-    if ( TiCC::split( line, parts ) != 3 ){
+    vector<string> parts = TiCC::split( line );
+    if ( parts.size() != 3 ){
       LOG << "invalid line in: '" << file << "' (expected 3 parts)" << endl;
       return false;
     }
@@ -80,7 +77,7 @@ bool Mblem::fill_ts_map( const string& file ){
   return true;
 }
 
-bool Mblem::init( const Configuration& config ) {
+bool Mblem::init( const TiCC::Configuration& config ) {
   LOG << "Initiating lemmatizer..." << endl;
   debug = 0;
   string val = config.lookUp( "debug", "mblem" );
@@ -126,7 +123,7 @@ bool Mblem::init( const Configuration& config ) {
     charFile = config.lookUp( "char_filter_file" );
   if ( !charFile.empty() ){
     charFile = prefix( config.configDir(), charFile );
-    filter = new Tokenizer::UnicodeFilter();
+    filter = new TiCC::UniFilter();
     filter->fill( charFile );
   }
 
@@ -139,8 +136,7 @@ bool Mblem::init( const Configuration& config ) {
 
   string one_one_tagS = config.lookUp( "one_one_tags", "mblem" );
   if ( !one_one_tagS.empty() ){
-    vector<string> tags;
-    TiCC::split_at( one_one_tagS, tags, "," );
+    vector<string> tags = TiCC::split_at( one_one_tagS, "," );
     for ( auto const& t : tags ){
       one_one_tags.insert( t );
     }
@@ -148,14 +144,22 @@ bool Mblem::init( const Configuration& config ) {
 
   string par = config.lookUp( "keep_case", "mblem" );
   if ( !par.empty() ){
-    keep_case = stringTo<bool>( par );
+    keep_case = TiCC::stringTo<bool>( par );
+  }
+
+  string cls = config.lookUp( "outputclass" );
+  if ( !cls.empty() ){
+    textclass = cls;
+  }
+  else {
+    textclass = "current";
   }
 
   string opts = config.lookUp( "timblOpts", "mblem" );
   if ( opts.empty() )
     opts = "-a1";
   //make it silent
-  opts += " +vs -vf";
+  opts += " +vs -vf -F TABBED";
   //Read in (igtree) data
   myLex = new Timbl::TimblAPI(opts);
   return myLex->GetInstanceBase(treeName);
@@ -169,35 +173,39 @@ Mblem::~Mblem(){
   delete mblemLog;
 }
 
-string Mblem::make_instance( const UnicodeString& in ) {
+string Mblem::make_instance( const icu::UnicodeString& in ) {
   if (debug) {
     LOG << "making instance from: " << in << endl;
   }
-  UnicodeString instance = "";
+  icu::UnicodeString instance = "";
   size_t length = in.length();
   for ( size_t i=0; i < history; i++) {
     size_t j = length - history + i;
     if (( i < history - length ) &&
 	(length<history))
-      instance += "= ";
+      instance += "=\t";
     else {
       instance += in[j];
-      instance += ' ';
+      instance += '\t';
     }
   }
   instance += "?";
-  string result = UnicodeToUTF8(instance);
+  string result = TiCC::UnicodeToUTF8(instance);
   if ( debug ){
     LOG << "inst: " << instance << endl;
   }
   return result;
 }
 
-void Mblem::addLemma( Word *word, const string& cls ){
-  KWargs args;
+void Mblem::addLemma( folia::Word *word, const string& cls ){
+  folia::KWargs args;
   args["set"]=tagset;
   args["class"]=cls;
-#pragma omp critical(foliaupdate)
+  if ( textclass != "current" ){
+    args["textclass"] = textclass;
+  }
+
+#pragma omp critical (foliaupdate)
   {
     try {
       word->addLemmaAnnotation( args );
@@ -266,10 +274,10 @@ void Mblem::makeUnique( ){
   }
 }
 
-void Mblem::getFoLiAResult( Word *word, const UnicodeString& uWord ){
+void Mblem::getFoLiAResult( folia::Word *word, const icu::UnicodeString& uWord ){
   if ( mblemResult.empty() ){
     // just return the word as a lemma
-    string result = UnicodeToUTF8( uWord );
+    string result = TiCC::UnicodeToUTF8( uWord );
     addLemma( word, result );
   }
   else {
@@ -281,27 +289,29 @@ void Mblem::getFoLiAResult( Word *word, const UnicodeString& uWord ){
 }
 
 
-void Mblem::addDeclaration( Document& doc ) const {
-#pragma omp critical (foliaupdate)
-  {
-    doc.declare( AnnotationType::LEMMA,
-		 tagset,
-		 "annotator='frog-mblem-" + version
-		 + "', annotatortype='auto', datetime='" + getTime() + "'");
-  }
+void Mblem::addDeclaration( folia::Document& doc ) const {
+  doc.declare( folia::AnnotationType::LEMMA,
+	       tagset,
+	       "annotator='frog-mblem-" + version
+	       + "', annotatortype='auto', datetime='" + getTime() + "'");
 }
 
-void Mblem::Classify( Word *sword ){
-  if ( sword->isinstance(PlaceHolder_t ) )
+void Mblem::Classify( folia::Word *sword ){
+  if ( sword->isinstance( folia::PlaceHolder_t ) )
     return;
-  UnicodeString uword;
+  icu::UnicodeString uword;
   string pos;
   string token_class;
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
   {
-    uword = sword->text();
+    uword = sword->text( textclass );
     pos = sword->pos();
-    token_class = sword->cls();
+    string txtcls = sword->textclass();
+    if ( txtcls == textclass ){
+      // so only use the word class is the textclass of the word
+      // matches the wanted text
+      token_class = sword->cls();
+    }
   }
   if (debug){
     LOG << "Classify " << uword << "(" << pos << ") ["
@@ -312,7 +322,7 @@ void Mblem::Classify( Word *sword ){
 
   if ( token_class == "ABBREVIATION" ){
     // We dont handle ABBREVIATION's so just take the word as such
-    string word = UnicodeToUTF8(uword);
+    string word = TiCC::UnicodeToUTF8(uword);
     addLemma( sword, word );
     return;
   }
@@ -322,15 +332,18 @@ void Mblem::Classify( Word *sword ){
     // we have to strip a few letters to get a lemma
     auto const& it2 = it1->second.find( token_class );
     if ( it2 != it1->second.end() ){
-      uword = UnicodeString( uword, 0, uword.length() - it2->second );
-      string word = UnicodeToUTF8(uword);
+      icu::UnicodeString uword2 = icu::UnicodeString( uword, 0, uword.length() - it2->second );
+      if ( uword2.isEmpty() ){
+	uword2 = uword;
+      }
+      string word = TiCC::UnicodeToUTF8(uword2);
       addLemma( sword, word );
       return;
     }
   }
   if ( one_one_tags.find(pos) != one_one_tags.end() ){
     // some tags are just taken as such
-    string word = UnicodeToUTF8(uword);
+    string word = TiCC::UnicodeToUTF8(uword);
     addLemma( sword, word );
     return;
   }
@@ -343,24 +356,24 @@ void Mblem::Classify( Word *sword ){
   getFoLiAResult( sword, uword );
 }
 
-void Mblem::Classify( const UnicodeString& uWord ){
+void Mblem::Classify( const icu::UnicodeString& uWord ){
   mblemResult.clear();
   string inst = make_instance(uWord);
   string classString;
   myLex->Classify( inst, classString );
-  if (debug){
+  if ( debug){
     LOG << "class: " << classString  << endl;
   }
   // 1st find all alternatives
   vector<string> parts;
-  int numParts = split_at( classString, parts, "|" );
+  int numParts = TiCC::split_at( classString, parts, "|" );
   if ( numParts < 1 ){
     LOG << "no alternatives found" << endl;
   }
   int index = 0;
   while ( index < numParts ) {
     string partS = parts[index++];
-    UnicodeString lemma;
+    icu::UnicodeString lemma;
     string restag;
     string::size_type pos = partS.find("+");
     if ( pos == string::npos ){
@@ -370,28 +383,28 @@ void Mblem::Classify( const UnicodeString& uWord ){
     }
     else {
       // some edit info available, like: WW(27)+Dgekomen+Ikomen
-      vector<string> edits;
-      size_t n = split_at( partS, edits, "+" );
-      if ( n < 1 )
+      vector<string> edits = TiCC::split_at( partS, "+" );
+      if ( edits.empty() ){
 	throw runtime_error( "invalid editstring: " + partS );
+      }
       restag = edits[0]; // the first one is the POS tag
 
-      UnicodeString insstr;
-      UnicodeString delstr;
-      UnicodeString prefix;
+      icu::UnicodeString insstr;
+      icu::UnicodeString delstr;
+      icu::UnicodeString prefix;
       for ( const auto& edit : edits ){
 	if ( edit == edits.front() ){
 	  continue;
 	}
 	switch ( edit[0] ){
 	case 'P':
-	  prefix = UTF8ToUnicode( edit.substr( 1 ) );
+	  prefix = TiCC::UnicodeFromUTF8( edit.substr( 1 ) );
 	  break;
 	case 'I':
-	  insstr = UTF8ToUnicode( edit.substr( 1 ) );
+	  insstr = TiCC::UnicodeFromUTF8( edit.substr( 1 ) );
 	  break;
 	case 'D':
-	  delstr =  UTF8ToUnicode( edit.substr( 1 ) );
+	  delstr = TiCC::UnicodeFromUTF8( edit.substr( 1 ) );
 	  break;
 	default:
 	  LOG << "Error: strange value in editstring: " << edit
@@ -422,7 +435,7 @@ void Mblem::Classify( const UnicodeString& uWord ){
 	LOG << "prefixpos = " << prefixpos << endl;
       }
       if (prefixpos >= 0) {
-	lemma = UnicodeString( uWord, 0L, prefixpos );
+	lemma = icu::UnicodeString( uWord, 0L, prefixpos );
 	prefixpos = prefixpos + prefix.length();
       }
       if (debug){
@@ -441,13 +454,13 @@ void Mblem::Classify( const UnicodeString& uWord ){
 	    lemma += uWord;
 	  }
 	  else {
-	    UnicodeString part = UnicodeString( uWord, prefixpos, uWord.length() - delstr.length() - prefixpos );
+	    icu::UnicodeString part = icu::UnicodeString( uWord, prefixpos, uWord.length() - delstr.length() - prefixpos );
 	    lemma += part + insstr;
 	  }
 	}
 	else if ( insstr.isEmpty() ){
 	  // no replacement, just take part after the prefix
-	  lemma += UnicodeString( uWord, prefixpos, uWord.length() ); // uWord;
+	  lemma += icu::UnicodeString( uWord, prefixpos, uWord.length() ); // uWord;
 	}
 	else {
 	  // but replace if possible
@@ -478,7 +491,7 @@ void Mblem::Classify( const UnicodeString& uWord ){
     if ( debug ){
       LOG << "appending lemma " << lemma << " and tag " << restag << endl;
     }
-    mblemResult.push_back( mblemData( UnicodeToUTF8(lemma), restag ) );
+    mblemResult.push_back( mblemData( TiCC::UnicodeToUTF8(lemma), restag ) );
   } // while
   if ( debug ) {
     LOG << "stored lemma and tag options: " << mblemResult.size()
