@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -29,12 +29,12 @@
 
 */
 
+#include "frog/mblem_mod.h"
+
 #include <string>
 #include <iostream>
-#include <sstream>
 #include <fstream>
 #include <vector>
-#include <map>
 
 #include "config.h"
 #include "ticcutils/LogStream.h"
@@ -44,21 +44,19 @@
 #include "libfolia/folia.h"
 #include "frog/ucto_tokenizer_mod.h"
 #include "frog/cgn_tagger_mod.h"
-#include "frog/mblem_mod.h"
 
 using namespace std;
 using namespace Timbl;
-using namespace TiCC;
 using namespace Tagger;
 
-LogStream my_default_log( cerr, "", StampMessage ); // fall-back
-LogStream *theErrLog = &my_default_log;  // fill the externals
+TiCC::LogStream my_default_log( cerr, "", StampMessage ); // fall-back
+TiCC::LogStream *theErrLog = &my_default_log;  // fill the externals
 
 vector<string> fileNames;
 bool useTagger = true;
 bool useTokenizer = true;
 
-Configuration configuration;
+TiCC::Configuration configuration;
 static string configDir = string(SYSCONF_PATH) + "/" + PACKAGE + "/nld/";
 static string configFileName = configDir + "frog.cfg";
 
@@ -168,7 +166,7 @@ void Test( istream& in ){
       if ( useTagger ){
 	vector<TagResult> tagrv = tagger.tagLine( s );
 	for ( const auto& tr : tagrv ){
-	  UnicodeString uWord = folia::UTF8ToUnicode(tr.word());
+	  icu::UnicodeString uWord = TiCC::UnicodeFromUTF8(tr.word());
 	  myMblem.Classify( uWord );
 	  myMblem.filterTag( tr.assignedTag() );
 	  vector<pair<string,string> > res = myMblem.getResult();
@@ -181,10 +179,9 @@ void Test( istream& in ){
 	}
       }
       else {
-	vector<string> parts;
-	TiCC::split( s, parts );
+	vector<string> parts = TiCC::split( s );
 	for ( const auto& w : parts ){
-	  UnicodeString uWord = folia::UTF8ToUnicode(w);
+	  icu::UnicodeString uWord = TiCC::UnicodeFromUTF8(w);
 	  myMblem.Classify( uWord );
 	  vector<pair<string,string> > res = myMblem.getResult();
 	  string line = w + "\t";
@@ -205,7 +202,7 @@ void Test( istream& in ){
 
 int main(int argc, char *argv[]) {
   std::ios_base::sync_with_stdio(false);
-  cerr << "mblem " << VERSION << " (c) CLTS, ILK 2014 - 2017" << endl;
+  cerr << "mblem " << VERSION << " (c) CLTS, ILK 2014 - 2018" << endl;
   cerr << "CLST  - Centre for Language and Speech Technology,"
        << "Radboud University" << endl
        << "ILK   - Induction of Linguistic Knowledge Research Group,"
