@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -29,26 +29,25 @@
 
 */
 
+#include "frog/mbma_brackets.h"
+
+#include <cassert>
 #include <string>
 #include <vector>
 #include <list>
 #include <iostream>
-#include "ucto/unicode.h"
 #include "ticcutils/Configuration.h"
-#include "ticcutils/LogStream.h"
 #include "ticcutils/StringOps.h"
 #include "ticcutils/PrettyPrint.h"
 #include "libfolia/folia.h"
 #include "frog/clex.h"
 #include "frog/mbma_rule.h"
 #include "frog/mbma_mod.h"
-#include "frog/mbma_brackets.h"
 
 using namespace std;
-using namespace TiCC;
-using namespace folia;
+using TiCC::operator<<;
 
-#define LOG *Log(myLog)
+#define LOG *TiCC::Log(myLog)
 
 string toString( const Compound::Type& ct ){
   switch ( ct ){
@@ -235,7 +234,7 @@ ostream& operator<<( ostream& os, const Status& st ){
   return os;
 }
 
-BracketLeaf::BracketLeaf( const RulePart& p, int flag, LogStream& l ):
+BracketLeaf::BracketLeaf( const RulePart& p, int flag, TiCC::LogStream& l ):
   BaseBracket(p.ResultClass, p.RightHand, flag, l ),
   glue(false),
   morph(p.morpheme )
@@ -283,9 +282,9 @@ BracketLeaf::BracketLeaf( const RulePart& p, int flag, LogStream& l ):
 }
 
 BracketLeaf::BracketLeaf( CLEX::Type t,
-			  const UnicodeString& us,
+			  const icu::UnicodeString& us,
 			  int flag,
-			  LogStream& l ):
+			  TiCC::LogStream& l ):
   BaseBracket( t, vector<CLEX::Type>(), flag, l ),
   morph( us )
 {
@@ -297,8 +296,9 @@ BracketLeaf::BracketLeaf( CLEX::Type t,
 BracketNest::BracketNest( CLEX::Type t,
 			  Compound::Type c,
 			  int flag,
-			  LogStream& l ): BaseBracket( t, flag, l ),
-					  _compound( c )
+			  TiCC::LogStream& l ):
+  BaseBracket( t, flag, l ),
+  _compound( c )
 {
   _status = Status::COMPLEX;
 }
@@ -314,17 +314,17 @@ BracketNest::~BracketNest(){
   }
 }
 
-UnicodeString BaseBracket::put( bool full ) const {
-  UnicodeString result = "[err?]";
+icu::UnicodeString BaseBracket::put( bool full ) const {
+  icu::UnicodeString result = "[err?]";
   if ( full ){
-    UnicodeString s = UTF8ToUnicode(toString(cls));
+    icu::UnicodeString s = TiCC::UnicodeFromUTF8(toString(cls));
     result += s;
   }
   return result;
 }
 
-UnicodeString BracketLeaf::put( bool full ) const {
-  UnicodeString result;
+icu::UnicodeString BracketLeaf::put( bool full ) const {
+  icu::UnicodeString result;
   if ( !morph.isEmpty() ){
     result += "[";
     result += morph;
@@ -332,30 +332,39 @@ UnicodeString BracketLeaf::put( bool full ) const {
   }
   if ( full ){
     if ( orig.empty() ){
-      result += UTF8ToUnicode(inflect);
+      icu::UnicodeString s = TiCC::UnicodeFromUTF8(toString(cls));
+      if ( s == "/" ){
+	result += s + TiCC::UnicodeFromUTF8(inflect);
+      }
+      else {
+	result += s + "/" + TiCC::UnicodeFromUTF8(inflect);
+      }
     }
     else {
-      result += UTF8ToUnicode(orig);
+      result += TiCC::UnicodeFromUTF8(orig);
     }
   }
   return result;
 }
 
-UnicodeString BracketNest::put( bool full ) const {
-  UnicodeString result = "[ ";
+icu::UnicodeString BracketNest::put( bool full ) const {
+  icu::UnicodeString result = "[ ";
   for ( auto const& it : parts ){
-    UnicodeString m = it->put( full );
+    icu::UnicodeString m = it->put( full );
     if ( !m.isEmpty() ){
-      result +=  m + " ";
+      result += m + " ";
+      // if (&it != &parts.back() ){
+      // 	result += " ";
+      // }
     }
   }
   result += "]";
   if ( full ){
     if ( cls != CLEX::UNASS ){
-      result += UTF8ToUnicode(toString(cls));
+      result += TiCC::UnicodeFromUTF8(toString(cls));
     }
     if ( _compound != Compound::Type::NONE ){
-      result += " " + UTF8ToUnicode(toString(_compound)) + "-compound";
+      result += " " + TiCC::UnicodeFromUTF8(toString(_compound)) + "-compound";
     }
   }
   return result;
@@ -379,7 +388,10 @@ ostream& operator<< ( ostream& os, const BaseBracket *c ){
 void prettyP( ostream& os, const list<BaseBracket*>& v ){
   os << "[";
   for ( auto const& it : v ){
-    os << it << " ";
+    os << it;
+    if ( &it != &v.back() ){
+      os << " ";
+    }
   }
   os << "]";
 }
@@ -442,7 +454,7 @@ bool BracketNest::testMatch( list<BaseBracket*>& result,
   return true;
 }
 
-Compound::Type construct( const vector<CLEX::Type> tags ){
+Compound::Type construct( const vector<CLEX::Type>& tags ){
   string s;
   for ( const auto& t : tags ){
     s += toString( t );
@@ -462,6 +474,7 @@ Compound::Type construct( const CLEX::Type tag1, const CLEX::Type tag2 ){
   return construct( v );
 }
 
+bool TEST = 1;
 Compound::Type BracketNest::getCompoundType(){
   if ( debugFlag > 5 ){
     LOG << "get compoundType: " << this << endl;
@@ -485,12 +498,16 @@ Compound::Type BracketNest::getCompoundType(){
       LOG << "tag2 :" << tag2 << " stat2: " << st2 << " cp2: " << cp2 << endl;
     }
     if ( st1 != Status::FAILED
-	 && st2 != Status::FAILED ){
-      if ( tag1 == CLEX::N
-	   && st1 != Status::DERIVATIONAL
-	   && st1 != Status::PARTICLE
-	   && st1 != Status::PARTICIPLE ){
-	if ( st2 == Status::STEM ){
+	 && st2 != Status::FAILED
+	 && st1 != Status::PARTICLE
+	 && st1 != Status::PARTICIPLE ){
+      switch ( tag1 ){
+      case CLEX::N:
+      case CLEX::A:
+	if ( st1 == Status::DERIVATIONAL ){
+	  compound = cp2;
+	}
+	else if ( st2 == Status::STEM ){
 	  compound = construct( tag1, tag2 );
 	}
 	else if ( st2 == Status::DERIVATIONAL
@@ -498,45 +515,31 @@ Compound::Type BracketNest::getCompoundType(){
 		  || st2 == Status::INFLECTION ){
 	  compound = cp1;
 	}
-      }
-      else if ( tag1 == CLEX::A
-		&& st1 != Status::PARTICLE
-		&& st1 != Status::PARTICIPLE ){
+	break;
+      case  CLEX::B:
 	if ( st2 == Status::STEM ){
 	  compound = construct( tag1, tag2 );
 	}
-	else if ( st2 == Status::DERIVATIONAL
-		  || st2 == Status::INFO
-		  || st2 == Status::INFLECTION ){
+	break;
+      case CLEX::P:
+	if ( st2 == Status::STEM ){
+	  compound = construct( tag1, tag2 );
+	}
+	else if ( tag2 == CLEX::NEUTRAL
+		  || tag2 == CLEX::UNASS ){
 	  compound = cp1;
 	}
-      }
-      else if ( tag1 == CLEX::B ){
-	if ( st2 == Status::STEM ){
-	  compound = construct( tag1, tag2 );
+	break;
+      case CLEX::V:
+	if ( st1 == Status::DERIVATIONAL ){
+	  compound = cp2;
 	}
-      }
-      else if ( tag1 == CLEX::P ){
-	if ( st2 == Status::STEM ){
-	  compound = construct( tag1, tag2 );
-	}
-	else if ( tag2 == CLEX::NEUTRAL || tag2 == CLEX::UNASS ){
-	  compound = cp1;
-	}
-      }
-      else if ( tag1 == CLEX::V ){
-	if ( st1 != Status::PARTICLE
-	     && st1 != Status::PARTICIPLE ){
-	  if ( st1 == Status::DERIVATIONAL ){
-	    compound = cp2;
-	  }
-	  else if ( st2 == Status::STEM ){
-	    compound = construct( tag1, tag2 );
-	  }
-	}
-	else if ( st2 == Status::COMPLEX ) {
+	else if ( st2 == Status::STEM ){
 	  compound = construct( tag1, tag2 );
 	}
+	break;
+      default:
+	break;
       }
     }
   }
@@ -561,8 +564,13 @@ Compound::Type BracketNest::getCompoundType(){
 	 && st3 != Status::FAILED
 	 && st1 != Status::PARTICLE
 	 && st1 != Status::PARTICIPLE ){
-      if ( tag1 == CLEX::N ){
-	if ( st2 == Status::STEM &&
+      switch ( tag1 ){
+      case CLEX::N:
+	if ( st2 == Status::STEM && tag2 == CLEX::N
+	     && st3 == Status::STEM && tag3 == CLEX::N ){
+	  compound = Compound::Type::NNN;
+	}
+	else if ( st1 != Status::DERIVATIONAL && st2 == Status::STEM &&
 	     ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ){
 	  compound = construct( tag1, tag2 );
 	}
@@ -603,11 +611,8 @@ Compound::Type BracketNest::getCompoundType(){
 	    compound = Compound::Type::NN;
 	  }
 	}
-	else if ( tag2 == CLEX::N && tag3 == CLEX::N ){
-	  compound = Compound::Type::NNN;
-	}
-      }
-      else if ( tag1 == CLEX::A ){
+	break;
+      case CLEX::A:
 	if ( st2 == Status::STEM &&
 	     ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ){
 	  compound = construct( tag1, tag2 );
@@ -625,8 +630,8 @@ Compound::Type BracketNest::getCompoundType(){
 	    compound = cp1;
 	  }
 	}
-      }
-      else if ( tag1 == CLEX::P ){
+	break;
+      case CLEX::P:
 	if ( st2 == Status::STEM &&
 	     ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ){
 	  compound = construct( tag1, tag2 );
@@ -640,30 +645,35 @@ Compound::Type BracketNest::getCompoundType(){
 	else if ( st3 == Status::DERIVATIONAL ){
 	  compound = construct( tag1, tag3 );
 	}
-      }
-      else if ( tag1 == CLEX::B && st1 == Status::STEM ){
-      	if ( ( st2 == Status::STEM
-	       && ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ) ){
-	  compound = construct( tag1, tag2 );
-	}
-      	else if ( st2 == Status::COMPLEX ){
-	  if ( tag2 == CLEX::N ){
-	    compound = Compound::Type::BN;
+	break;
+      case CLEX::B:
+	if ( st1 == Status::STEM ){
+	  if ( ( st2 == Status::STEM
+		 && ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ) ){
+	    compound = construct( tag1, tag2 );
 	  }
-	  else {
-	    compound = cp2;
+	  else if ( st2 == Status::COMPLEX ){
+	    if ( tag2 == CLEX::N ){
+	      compound = Compound::Type::BN;
+	    }
+	    else {
+	      compound = cp2;
+	    }
 	  }
 	}
-      }
-      else if ( tag1 == CLEX::V ){
+	break;
+      case CLEX::V:
       	if ( st2 == Status::STEM &&
 	     ( st3 == Status::INFLECTION || tag3 == CLEX::NEUTRAL ) ){
 	  compound = construct( tag1, tag2 );
 	}
       	else if ( st3 == Status::STEM &&
-	     ( st2 == Status::INFLECTION ) ){
+		  ( st2 == Status::INFLECTION ) ){
 	  compound = construct( tag1, tag3 );
 	}
+	break;
+      default:
+	break;
       }
     }
   }
@@ -674,16 +684,16 @@ Compound::Type BracketNest::getCompoundType(){
   return compound;
 }
 
-Morpheme *BracketLeaf::createMorpheme( Document *doc ) const {
+folia::Morpheme *BracketLeaf::createMorpheme( folia::Document *doc ) const {
   string desc;
   int cnt = 0;
   return createMorpheme( doc, desc, cnt );
 }
 
-Morpheme *BracketLeaf::createMorpheme( Document *doc,
-				       string& desc,
-				       int& cnt ) const {
-  Morpheme *result = 0;
+folia::Morpheme *BracketLeaf::createMorpheme( folia::Document *doc,
+					      string& desc,
+					      int& cnt ) const {
+  folia::Morpheme *result = 0;
   desc.clear();
   string::size_type pos = orig.find( "^" );
   bool glue = ( pos != string::npos );
@@ -692,20 +702,17 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
   }
   else if ( _status == Status::STEM
 	    || ( _status == Status::DERIVATIONAL && glue ) ){
-    KWargs args;
-    args["set"] = Mbma::mbma_tagset;
-    args["class"] = "stem";
-    result = new Morpheme( args, doc );
-    args.clear();
-    string out = UnicodeToUTF8(morph);
+    string out = TiCC::UnicodeToUTF8(morph);
     if ( out.empty() ){
       throw logic_error( "stem has empty morpheme" );
     }
-    args["value"] = out;
-    TextContent *t = new TextContent( args );
-#pragma omp critical(foliaupdate)
+    folia::KWargs args;
+    args["set"] = Mbma::mbma_tagset;
+    args["class"] = "stem";
+#pragma omp critical (foliaupdate)
     {
-      result->append( t );
+      result = new folia::Morpheme( args, doc );
+      result->settext( out );
     }
     ++cnt;
     args.clear();
@@ -721,58 +728,48 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
       args["class"] = toString( tag() );
       desc = "[" + out + "]" + CLEX::get_tDescr( tag() ); // spread the word upwards!
     }
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
     {
       result->addPosAnnotation( args );
     }
   }
   else if ( _status == Status::PARTICLE ){
-    KWargs args;
-    args["set"] = Mbma::mbma_tagset;
-    args["class"] = "particle";
-    result = new Morpheme( args, doc );
-    args.clear();
-    string out = UnicodeToUTF8(morph);
+    string out = TiCC::UnicodeToUTF8(morph);
     if ( out.empty() ){
       throw logic_error( "particle has empty morpheme" );
     }
-    args["value"] = out;
-    TextContent *t = new TextContent( args );
-#pragma omp critical(foliaupdate)
+    folia::KWargs args;
+    args["set"] = Mbma::mbma_tagset;
+    args["class"] = "particle";
+#pragma omp critical (foliaupdate)
     {
-      result->append( t );
+      result = new folia::Morpheme( args, doc );
+      result->settext( out );
     }
     ++cnt;
     args.clear();
     args["set"] = Mbma::clex_tagset;
     args["class"] = toString( tag() );
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
     {
       result->addPosAnnotation( args );
     }
     desc = "[" + out + "]"; // spread the word upwards! maybe add 'part' ??
   }
   else if ( _status == Status::INFLECTION ){
-    KWargs args;
-    args["class"] = "inflection";
-    args["set"] = Mbma::mbma_tagset;
-    result = new Morpheme( args, doc );
-    args.clear();
-    string out = UnicodeToUTF8(morph);
-    if ( out.empty() ){
-      out = inflect;
-    }
-    else {
+    string out = TiCC::UnicodeToUTF8(morph);
+    if ( !out.empty() ){
       desc = "[" + out + "]";
     }
-    if ( out.empty() ){
-      throw logic_error( "Inflection and morpheme empty" );
-    }
-    args["value"] = out;
-    TextContent *t = new TextContent( args );
-#pragma omp critical(foliaupdate)
+    folia::KWargs args;
+    args["class"] = "inflection";
+    args["set"] = Mbma::mbma_tagset;
+#pragma omp critical (foliaupdate)
     {
-      result->append( t );
+      result = new folia::Morpheme( args, doc );
+      if ( !out.empty() ){
+	result->settext( out );
+      }
     }
     ++cnt;
     args.clear();
@@ -784,9 +781,9 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
 	  // happens sometimes when there is fawlty data
 	  args["class"] = d;
 	  desc += "/" + d;
-	  folia::Feature *feat = new folia::Feature( args );
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
 	  {
+	    folia::Feature *feat = new folia::Feature( args );
 	    result->append( feat );
 	  }
 	}
@@ -796,7 +793,11 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
   else if ( _status == Status::DERIVATIONAL
 	    || _status == Status::PARTICIPLE
 	    || _status == Status::FAILED ){
-    KWargs args;
+    string out = TiCC::UnicodeToUTF8(morph);
+    if ( out.empty() ){
+      throw logic_error( "Derivation with empty morpheme" );
+    }
+    folia::KWargs args;
     if ( _status == Status::DERIVATIONAL ){
       args["class"] = "affix";
     }
@@ -807,18 +808,10 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
       args["class"] = "derivational";
     }
     args["set"] = Mbma::mbma_tagset;
-    result = new Morpheme( args, doc );
-    args.clear();
-    string out = UnicodeToUTF8(morph);
-    if ( out.empty() ){
-      LOG << "problem: " << this << endl;
-      throw logic_error( "Derivation with empty morpheme" );
-    }
-    args["value"] = out;
-    TextContent *t = new TextContent( args );
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
     {
-      result->append( t );
+      result = new folia::Morpheme( args, doc );
+      result->settext( out );
     }
     ++cnt;
     desc = "[" + out + "]"; // pass it up!
@@ -834,7 +827,7 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
     args.clear();
     args["subset"] = "structure";
     args["class"]  = desc;
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
     {
       folia::Feature *feat = new folia::Feature( args );
       result->append( feat );
@@ -842,16 +835,19 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
     args.clear();
 //     args["set"] = Mbma::clex_tagset;
 //     args["class"] = orig;
-// #pragma omp critical(foliaupdate)
+// #pragma omp critical (foliaupdate)
 //     {
 //       result->addPosAnnotation( args );
 //     }
   }
   else if ( _status == Status::INFO ){
-    KWargs args;
+    folia::KWargs args;
     args["class"] = "inflection";
     args["set"] = Mbma::mbma_tagset;
-    result = new Morpheme( args, doc );
+#pragma omp critical (foliaupdate)
+    {
+      result = new folia::Morpheme( args, doc );
+    }
     args.clear();
     args["subset"] = "inflection";
     for ( const auto& inf : inflect ){
@@ -861,9 +857,9 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
 	  // happens sometimes when there is fawlty data
 	  desc += "/" + d;
 	  args["class"] = d;
-	  folia::Feature *feat = new folia::Feature( args );
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
 	  {
+	    folia::Feature *feat = new folia::Feature( args );
 	    result->append( feat );
 	  }
 	}
@@ -873,36 +869,39 @@ Morpheme *BracketLeaf::createMorpheme( Document *doc,
   return result;
 }
 
-Morpheme *BracketNest::createMorpheme( Document *doc ) const {
+folia::Morpheme *BracketNest::createMorpheme( folia::Document *doc ) const {
   string desc;
   int cnt = 0;
   return createMorpheme( doc, desc, cnt );
 }
 
-Morpheme *BracketNest::createMorpheme( Document *doc,
-				       string& desc,
-				       int& cnt ) const {
-  KWargs args;
+folia::Morpheme *BracketNest::createMorpheme( folia::Document *doc,
+					      string& desc,
+					      int& cnt ) const {
+  folia::Morpheme *result = 0;
+  folia::KWargs args;
   args["class"] = "complex";
   args["set"] = Mbma::mbma_tagset;
-  Morpheme *result = new Morpheme( args, doc );
-  string mor;
+#pragma omp critical (foliaupdate)
+  {
+    result = new folia::Morpheme( args, doc );
+  }
   cnt = 0;
   desc.clear();
-  vector<Morpheme*> stack;
+  vector<folia::Morpheme*> stack;
   for ( auto const& it : parts ){
     string deeper_desc;
     int deep_cnt = 0;
-    Morpheme *m = it->createMorpheme( doc,
-				      deeper_desc,
-				      deep_cnt );
+    folia::Morpheme *m = it->createMorpheme( doc,
+					     deeper_desc,
+					     deep_cnt );
     if ( it->status() == Status::DERIVATIONAL
 	 || it->status() == Status::PARTICIPLE ){
       if ( !it->original().empty() ){
 	args.clear();
 	args["subset"] = "applied_rule";
 	args["class"] = it->original();
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
 	{
 	  folia::Feature *feat = new folia::Feature( args );
 	  result->append( feat );
@@ -922,7 +921,7 @@ Morpheme *BracketNest::createMorpheme( Document *doc,
   args.clear();
   args["subset"] = "structure";
   args["class"]  = desc;
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
   {
     folia::Feature *feat = new folia::Feature( args );
     result->append( feat );
@@ -930,8 +929,8 @@ Morpheme *BracketNest::createMorpheme( Document *doc,
   args.clear();
   args["set"] = Mbma::clex_tagset;
   args["class"] = toString( tag() );
-  PosAnnotation *pos = 0;
-#pragma omp critical(foliaupdate)
+  folia::PosAnnotation *pos = 0;
+#pragma omp critical (foliaupdate)
   {
     pos = result->addPosAnnotation( args );
   }
@@ -940,13 +939,13 @@ Morpheme *BracketNest::createMorpheme( Document *doc,
     args.clear();
     args["subset"] = "compound";
     args["class"]  = toString(ct);
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
     {
       folia::Feature *feat = new folia::Feature( args );
       pos->append( feat );
     }
   }
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
   for ( const auto& s : stack ){
     result->append( s );
   }
@@ -1009,8 +1008,7 @@ void BracketNest::resolveNouns( ){
 	 && ( (*it)->tag() == CLEX::N && (*it)->status() == Status::STEM )
 	 && (*it)->RightHand.size() == 0 ){
       Compound::Type newt = Compound::Type::NN;
-      if ( (*prev)->compound() == Compound::Type::NN
-	   || (*prev)->compound() == Compound::Type::NN ){
+      if ( (*prev)->compound() == Compound::Type::NN ){
 	newt = Compound::Type::NNN;
       }
       BaseBracket *tmp = new BracketNest( CLEX::N, newt, debugFlag, myLog );
