@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -29,37 +29,39 @@
 
 */
 
+#include "frog/mbma_mod.h"
+
 #include <cstdlib>
 #include <string>
 #include <set>
 #include <iostream>
 #include <algorithm>
 #include <fstream>
-#include <sstream>
+
 #include "timbl/TimblAPI.h"
 
-#include "ucto/unicode.h"
 #include "ticcutils/Configuration.h"
-#include "frog/Frog.h"
-#include "frog/mbma_mod.h"
+#include "ticcutils/StringOps.h"
+#include "ticcutils/PrettyPrint.h"
+#include "ticcutils/Unicode.h"
+#include "frog/Frog-util.h"
 
 using namespace std;
-using namespace folia;
-using namespace TiCC;
+using TiCC::operator<<;
 
 const long int LEFT =  6; // left context
 const long int RIGHT = 6; // right context
 
-#define LOG *Log(mbmaLog)
+#define LOG *TiCC::Log(mbmaLog)
 
-Mbma::Mbma(LogStream * logstream):
+Mbma::Mbma( TiCC::LogStream * logstream):
   MTreeFilename( "dm.igtree" ),
   MTree(0),
-  transliterator(0),
   filter(0),
+  filter_diac(false),
   doDeepMorph(false)
 {
-  mbmaLog = new LogStream( logstream, "mbma-" );
+  mbmaLog = new TiCC::LogStream( logstream, "mbma-" );
 }
 
 // define the statics
@@ -70,7 +72,6 @@ string Mbma::clex_tagset = "http://ilk.uvt.nl/folia/sets/frog-mbpos-clex";
 
 Mbma::~Mbma() {
   cleanUp();
-  delete transliterator;
   delete filter;
   delete mbmaLog;
 }
@@ -82,7 +83,7 @@ void Mbma::init_cgn( const string& main, const string& sub ) {
     string line;
     while( getline( tc, line) ) {
       vector<string> tmp;
-      size_t num = split_at(line, tmp, " ");
+      size_t num = TiCC::split_at(line, tmp, " ");
       if ( num < 2 ){
 	LOG << "splitting '" << line << "' failed" << endl;
 	throw ( runtime_error("panic") );
@@ -91,14 +92,14 @@ void Mbma::init_cgn( const string& main, const string& sub ) {
     }
   }
   else {
-    throw ( runtime_error( "unable to open:" + sub ) );
+    throw ( runtime_error( "unable to open: " + main ) );
   }
   ifstream tc1( sub );
   if ( tc1 ){
     string line;
     while( getline(tc1, line) ) {
       vector<string> tmp;
-      size_t num = split_at(line, tmp, " ");
+      size_t num = TiCC::split_at(line, tmp, " ");
       if ( num == 2 ){
 	TAGconv.insert( make_pair( tmp[0], tmp[1] ) );
       }
@@ -109,18 +110,7 @@ void Mbma::init_cgn( const string& main, const string& sub ) {
   }
 }
 
-Transliterator *Mbma::init_trans( ){
-  UErrorCode stat = U_ZERO_ERROR;
-  Transliterator *t = Transliterator::createInstance( "NFD; [:M:] Remove; NFC",
-						      UTRANS_FORWARD,
-						      stat );
-  if ( U_FAILURE( stat ) ){
-    throw runtime_error( "initFilter FAILED !" );
-  }
-  return t;
-}
-
-bool Mbma::init( const Configuration& config ) {
+bool Mbma::init( const TiCC::Configuration& config ) {
   LOG << "Initiating morphological analyzer..." << endl;
   debugFlag = 0;
   string val = config.lookUp( "debug", "mbma" );
@@ -181,7 +171,7 @@ bool Mbma::init( const Configuration& config ) {
   }
   if ( !charFile.empty() ){
     charFile = prefix( config.configDir(), charFile );
-    filter = new Tokenizer::UnicodeFilter();
+    filter = new TiCC::UniFilter();
     filter->fill( charFile );
   }
   string tfName = config.lookUp( "treeFile", "mbma" );
@@ -191,11 +181,17 @@ bool Mbma::init( const Configuration& config ) {
   MTreeFilename = prefix( config.configDir(), tfName );
   string dof = config.lookUp( "filter_diacritics", "mbma" );
   if ( !dof.empty() ){
-    bool b = stringTo<bool>( dof );
-    if ( b ){
-      transliterator = init_trans();
-    }
+    filter_diac = TiCC::stringTo<bool>( dof );
   }
+
+  string cls = config.lookUp( "outputclass" );
+  if ( !cls.empty() ){
+    textclass = cls;
+  }
+  else {
+    textclass = "current";
+  }
+
   //Read in (igtree) data
   string opts = config.lookUp( "timblOpts", "mbma" );
   if ( opts.empty() ){
@@ -213,14 +209,14 @@ void Mbma::cleanUp(){
   clearAnalysis();
 }
 
-vector<string> Mbma::make_instances( const UnicodeString& word ){
+vector<string> Mbma::make_instances( const icu::UnicodeString& word ){
   vector<string> insts;
   insts.reserve( word.length() );
   for ( long i=0; i < word.length(); ++i ) {
     if (debugFlag > 10){
       LOG << "itt #:" << i << endl;
     }
-    UnicodeString inst;
+    icu::UnicodeString inst;
     for ( long j=i ; j <= i + RIGHT + LEFT; ++j ) {
       if (debugFlag > 10){
 	LOG << " " << j-LEFT << ": ";
@@ -242,7 +238,7 @@ vector<string> Mbma::make_instances( const UnicodeString& word ){
       }
     }
     inst += "?";
-    insts.push_back( UnicodeToUTF8(inst) );
+    insts.push_back( TiCC::UnicodeToUTF8(inst) );
   }
   return insts;
 }
@@ -267,13 +263,13 @@ string find_class( unsigned int step,
 vector<vector<string> > generate_all_perms( const vector<string>& classes ){
   // determine all alternative analyses, remember the largest
   // and store every part in a vector of string vectors
-  int largest_anal=1;
+  size_t largest_anal=1;
   vector<vector<string> > classParts;
   classParts.reserve( classes.size() );
   for ( const auto& cl : classes ){
-    vector<string> parts;
-    int num = split_at( cl, parts, "|" );
-    if ( num > 0 ){
+    vector<string> parts = TiCC::split_at( cl, "|" );
+    size_t num = parts.size();
+    if ( num > 1 ){
       classParts.push_back( parts );
       if ( num > largest_anal ){
 	largest_anal = num;
@@ -290,7 +286,7 @@ vector<vector<string> > generate_all_perms( const vector<string>& classes ){
   // now expand the result
   vector<vector<string> > result;
   result.reserve( largest_anal );
-  for ( int step=0; step < largest_anal; ++step ){
+  for ( size_t step=0; step < largest_anal; ++step ){
     vector<string> item;
     for ( const auto& cp : classParts ){
       item.push_back( find_class( step, cp, largest_anal ) );
@@ -308,23 +304,18 @@ void Mbma::clearAnalysis(){
 }
 
 Rule* Mbma::matchRule( const std::vector<std::string>& ana,
-		       const UnicodeString& word ){
+		       const icu::UnicodeString& word ){
   Rule *rule = new Rule( ana, word, *mbmaLog, debugFlag );
   if ( rule->performEdits() ){
     rule->reduceZeroNodes();
     if ( debugFlag ){
       LOG << "after reduction: " << rule << endl;
     }
-#ifdef EXPERIMENT
-    rule->resolveBrackets( doDeepMorph );
-#endif
     rule->resolve_inflections();
     if ( debugFlag ){
       LOG << "after resolving: " << rule << endl;
     }
-#ifndef EXPERIMENT
     rule->resolveBrackets( doDeepMorph );
-#endif
     rule->getCleanInflect();
     if ( debugFlag ){
       LOG << "1 added Inflection: " << rule << endl;
@@ -340,11 +331,12 @@ Rule* Mbma::matchRule( const std::vector<std::string>& ana,
   }
 }
 
-vector<Rule*> Mbma::execute( const UnicodeString& word,
+vector<Rule*> Mbma::execute( const icu::UnicodeString& word,
 			     const vector<string>& classes ){
   vector<vector<string> > allParts = generate_all_perms( classes );
   if ( debugFlag ){
-    string out = "alternatives: word=" + UnicodeToUTF8(word) + ", classes=<";
+    string out = "alternatives: word="
+      + TiCC::UnicodeToUTF8(word) + ", classes=<";
     for ( const auto& cls : classes ){
       out += cls + ",";
     }
@@ -364,12 +356,12 @@ vector<Rule*> Mbma::execute( const UnicodeString& word,
   return accepted;
 }
 
-void Mbma::addMorph( Word *word,
+void Mbma::addMorph( folia::Word *word,
 		     const vector<string>& morphs ) const {
-  KWargs args;
+  folia::KWargs args;
   args["set"] = mbma_tagset;
-  MorphologyLayer *ml;
-#pragma omp critical(foliaupdate)
+  folia::MorphologyLayer *ml;
+#pragma omp critical (foliaupdate)
   {
     try {
       ml = word->addMorphologyLayer( args );
@@ -382,7 +374,7 @@ void Mbma::addMorph( Word *word,
   addMorph( ml, morphs );
 }
 
-void Mbma::addBracketMorph( Word *word,
+void Mbma::addBracketMorph( folia::Word *word,
 			    const string& wrd,
 			    const string& tag ) const {
   if (debugFlag){
@@ -395,9 +387,9 @@ void Mbma::addBracketMorph( Word *word,
   }
   else if ( head == "X" ) {
     // unanalysed, so trust the TAGGER
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
     {
-      const auto pos = word->annotation<PosAnnotation>( cgn_tagset );
+      const auto pos = word->annotation<folia::PosAnnotation>( cgn_tagset );
       head = pos->feat("head");
     }
     if (debugFlag){
@@ -406,7 +398,7 @@ void Mbma::addBracketMorph( Word *word,
     const auto tagIt = TAGconv.find( head );
     if ( tagIt == TAGconv.end() ) {
       // this should never happen
-      throw ValueError( "unknown head feature '" + head + "'" );
+      throw folia::ValueError( "unknown head feature '" + head + "'" );
     }
     celex_tag = tagIt->second;
     head = CLEX::get_tDescr(CLEX::toCLEX(tagIt->second));
@@ -415,10 +407,10 @@ void Mbma::addBracketMorph( Word *word,
     }
   }
 
-  KWargs args;
+  folia::KWargs args;
   args["set"] = mbma_tagset;
-  MorphologyLayer *ml;
-#pragma omp critical(foliaupdate)
+  folia::MorphologyLayer *ml;
+#pragma omp critical (foliaupdate)
   {
     try {
       ml = word->addMorphologyLayer( args );
@@ -429,18 +421,16 @@ void Mbma::addBracketMorph( Word *word,
     }
   }
   args["class"] = "stem";
-  Morpheme *result = new Morpheme( args, word->doc() );
-  args.clear();
-  args["value"] = wrd;
-  TextContent *t = new TextContent( args );
-#pragma omp critical(foliaupdate)
+  folia::Morpheme *result = 0;
+#pragma omp critical (foliaupdate)
   {
-    result->append( t );
+    result = new folia::Morpheme( args, word->doc() );
+    result->settext( wrd, textclass );
   }
   args.clear();
   args["subset"] = "structure";
   args["class"]  = "[" + wrd + "]" + head;
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
   {
     folia::Feature *feat = new folia::Feature( args );
     result->append( feat );
@@ -448,26 +438,26 @@ void Mbma::addBracketMorph( Word *word,
   args.clear();
   args["set"] = clex_tagset;
   args["class"] = celex_tag;
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
   {
     result->addPosAnnotation( args );
   }
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
   {
     ml->append( result );
   }
 }
 
-void Mbma::addBracketMorph( Word *word,
+void Mbma::addBracketMorph( folia::Word *word,
 			    const string& orig_word,
 			    const BracketNest *brackets ) const {
   if (debugFlag){
     LOG << "addBracketMorph(" << word << "," << orig_word << "," << brackets << ")" << endl;
   }
-  KWargs args;
+  folia::KWargs args;
   args["set"] = mbma_tagset;
-  MorphologyLayer *ml;
-#pragma omp critical(foliaupdate)
+  folia::MorphologyLayer *ml;
+#pragma omp critical (foliaupdate)
   {
     try {
       ml = word->addMorphologyLayer( args );
@@ -477,43 +467,34 @@ void Mbma::addBracketMorph( Word *word,
       throw;
     }
   }
-  Morpheme *m = 0;
+  folia::Morpheme *m = 0;
   try {
     m = brackets->createMorpheme( word->doc() );
   }
   catch( const exception& e ){
-    cerr << "createMorpheme failed: " << e.what() << endl;
+    LOG << "createMorpheme failed: " << e.what() << endl;
     throw;
   }
   if ( m ){
-    args.clear();
-    args["value"] = orig_word;
-    TextContent *t = new TextContent( args );
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
     {
-      m->append( t );
+      m->settext( orig_word, textclass );
       ml->append( m );
     }
   }
 }
 
-void Mbma::addMorph( MorphologyLayer *ml,
+void Mbma::addMorph( folia::MorphologyLayer *ml,
 		     const vector<string>& morphs ) const {
   for ( const auto& mor : morphs ){
-    KWargs args;
+    folia::KWargs args;
     args["set"] = mbma_tagset;
-    Morpheme *m = new Morpheme( args, ml->doc() );
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
     {
+      folia::Morpheme *m = new folia::Morpheme( args, ml->doc() );
+      m->settext( mor, textclass );
       ml->append( m );
     }
-    args.clear();
-    args["value"] = mor;
-    TextContent *t = new TextContent( args );
-#pragma omp critical(foliaupdate)
-    {
-      m->append( t );
-    }
   }
 }
 
@@ -534,7 +515,7 @@ void Mbma::filterHeadTag( const string& head ){
   map<string,string>::const_iterator tagIt = TAGconv.find( head );
   if ( tagIt == TAGconv.end() ) {
     // this should never happen
-    throw ValueError( "unknown head feature '" + head + "'" );
+    throw folia::ValueError( "unknown head feature '" + head + "'" );
   }
   string celex_tag = tagIt->second;
   if (debugFlag){
@@ -683,13 +664,13 @@ void Mbma::filterSubTags( const vector<string>& feats ){
   // we still might have doubles. (different Rule's yielding the same result)
   // reduce these
   //
-  map<UnicodeString, Rule*> unique;
+  map<icu::UnicodeString, Rule*> unique;
   for ( const auto& ait : highConf ){
-    UnicodeString tmp = ait->getKey( doDeepMorph );
+    icu::UnicodeString tmp = ait->getKey( doDeepMorph );
     unique[tmp] = ait;
   }
   // so now we have map of 'equal' analysis.
-  // create a set for revers lookup.
+  // create a set for reverse lookup.
   set<Rule*> uniqueAna;
   for ( auto const& uit : unique ){
     uniqueAna.insert( uit.second );
@@ -737,7 +718,8 @@ void Mbma::assign_compounds(){
   }
 }
 
-void Mbma::getFoLiAResult( Word *fword, const UnicodeString& uword ) const {
+void Mbma::getFoLiAResult( folia::Word *fword,
+			   const icu::UnicodeString& uword ) const {
   if ( analysis.size() == 0 ){
     // fallback option: use the word and pretend it's a morpheme ;-)
     if ( debugFlag ){
@@ -745,18 +727,18 @@ void Mbma::getFoLiAResult( Word *fword, const UnicodeString& uword ) const {
 		    << uword << endl;
     }
     if ( doDeepMorph ){
-      addBracketMorph( fword, UnicodeToUTF8(uword), "X" );
+      addBracketMorph( fword, TiCC::UnicodeToUTF8(uword), "X" );
     }
     else {
       vector<string> tmp;
-      tmp.push_back( UnicodeToUTF8(uword) );
+      tmp.push_back( TiCC::UnicodeToUTF8(uword) );
       addMorph( fword, tmp );
     }
   }
   else {
     for ( auto const& sit : analysis ){
       if ( doDeepMorph ){
-	addBracketMorph( fword, UnicodeToUTF8(uword), sit->brackets );
+	addBracketMorph( fword, TiCC::UnicodeToUTF8(uword), sit->brackets );
       }
       else {
 	addMorph( fword, sit->extract_morphemes() );
@@ -765,45 +747,36 @@ void Mbma::getFoLiAResult( Word *fword, const UnicodeString& uword ) const {
   }
 }
 
-void Mbma::addDeclaration( Document& doc ) const {
-#pragma omp critical(foliaupdate)
-  {
-    doc.declare( AnnotationType::MORPHOLOGICAL, mbma_tagset,
+void Mbma::addDeclaration( folia::Document& doc ) const {
+  doc.declare( folia::AnnotationType::MORPHOLOGICAL, mbma_tagset,
+	       "annotator='frog-mbma-" +  version +
+	       + "', annotatortype='auto', datetime='" + getTime() + "'");
+  if ( doDeepMorph ){
+    doc.declare( folia::AnnotationType::POS, clex_tagset,
 		 "annotator='frog-mbma-" +  version +
 		 + "', annotatortype='auto', datetime='" + getTime() + "'");
-    if ( doDeepMorph ){
-      doc.declare( AnnotationType::POS, clex_tagset,
-		   "annotator='frog-mbma-" +  version +
-		   + "', annotatortype='auto', datetime='" + getTime() + "'");
-    }
   }
 }
 
-UnicodeString Mbma::filterDiacritics( const UnicodeString& in ) const {
-  if ( transliterator ){
-    UnicodeString result = in;
-    transliterator->transliterate( result );
-    return result;
-  }
-  else {
-    return in;
-  }
-}
-
-void Mbma::Classify( Word* sword ){
-  if ( sword->isinstance(PlaceHolder_t) ){
+void Mbma::Classify( folia::Word* sword ){
+  if ( sword->isinstance(folia::PlaceHolder_t) ){
     return;
   }
-  UnicodeString uWord;
-  PosAnnotation *pos;
+  icu::UnicodeString uWord;
+  folia::PosAnnotation *pos;
   string head;
   string token_class;
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
   {
-    uWord = sword->text();
-    pos = sword->annotation<PosAnnotation>( cgn_tagset );
+    uWord = sword->text( textclass );
+    pos = sword->annotation<folia::PosAnnotation>( cgn_tagset );
     head = pos->feat("head");
-    token_class = sword->cls();
+    string txtcls = sword->textclass();
+    if ( txtcls == textclass ){
+      // so only use the word class is the textclass of the word
+      // matches the wanted text
+      token_class = sword->cls();
+    }
   }
   if (debugFlag ){
     LOG << "Classify " << uWord << "(" << pos << ") ["
@@ -812,10 +785,17 @@ void Mbma::Classify( Word* sword ){
   if ( filter ){
     uWord = filter->filter( uWord );
   }
+  string word_s = TiCC::UnicodeToUTF8( uWord );
+  vector<string> parts = TiCC::split( word_s );
+  word_s.clear();
+  for ( const auto& p : parts ){
+    word_s += p;
+  }
+  uWord = TiCC::UnicodeFromUTF8( word_s );
   if ( head == "LET" || head == "SPEC" || token_class == "ABBREVIATION" ){
     // take over the letter/word 'as-is'.
     //  also ABBREVIATION's aren't handled bij mbma-rules
-    string word = UnicodeToUTF8( uWord );
+    string word = TiCC::UnicodeToUTF8( uWord );
     if ( doDeepMorph ){
       addBracketMorph( sword, word, head );
     }
@@ -826,13 +806,13 @@ void Mbma::Classify( Word* sword ){
     }
   }
   else {
-    UnicodeString lWord = uWord;
+    icu::UnicodeString lWord = uWord;
     if ( head != "SPEC" ){
       lWord.toLower();
     }
     Classify( lWord );
     vector<string> featVals;
-#pragma omp critical(foliaupdate)
+#pragma omp critical (foliaupdate)
     {
       vector<folia::Feature*> feats = pos->select<folia::Feature>();
       featVals.reserve( feats.size() );
@@ -846,9 +826,12 @@ void Mbma::Classify( Word* sword ){
   }
 }
 
-void Mbma::Classify( const UnicodeString& word ){
+void Mbma::Classify( const icu::UnicodeString& word ){
   clearAnalysis();
-  UnicodeString uWord = filterDiacritics( word );
+  icu::UnicodeString uWord = word;
+  if ( filter_diac ){
+    uWord = TiCC::filter_diacritics( uWord );
+  }
   vector<string> insts = make_instances( uWord );
   vector<string> classes;
   classes.reserve( insts.size() );
