@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -29,12 +29,11 @@
 
 */
 
+#include "frog/mbma_mod.h"
+
 #include <string>
 #include <iostream>
-#include <sstream>
-#include <fstream>
 #include <vector>
-#include <map>
 
 #include "config.h"
 #include "ticcutils/LogStream.h"
@@ -44,15 +43,13 @@
 #include "libfolia/folia.h"
 #include "frog/ucto_tokenizer_mod.h"
 #include "frog/cgn_tagger_mod.h"
-#include "frog/mbma_mod.h"
 
 using namespace std;
 using namespace Timbl;
-using namespace TiCC;
 using namespace Tagger;
 
-LogStream my_default_log( cerr, "", StampMessage ); // fall-back
-LogStream *theErrLog = &my_default_log;  // fill the externals
+TiCC::LogStream my_default_log( cerr, "", StampMessage ); // fall-back
+TiCC::LogStream *theErrLog = &my_default_log;  // fill the externals
 
 vector<string> fileNames;
 bool useTagger = true;
@@ -60,7 +57,7 @@ bool useTokenizer = true;
 bool bulk = false;
 bool verbose = false;
 
-Configuration configuration;
+TiCC::Configuration configuration;
 static string configDir = string(SYSCONF_PATH) + "/" + PACKAGE + "/nld/";
 static string configFileName = configDir + "frog.cfg";
 
@@ -187,11 +184,9 @@ void Test( istream& in ){
       if ( useTagger ){
 	vector<TagResult> tagv = tagger.tagLine( s );
 	for ( const auto& tr : tagv ){
-	  UnicodeString uWord = folia::UTF8ToUnicode( tr.word() );
-	  vector<string> v;
-	  size_t num = TiCC::split_at_first_of( tr.assignedTag(),
-						v, "(,)" );
-	  if ( num < 1 ){
+	  icu::UnicodeString uWord = TiCC::UnicodeFromUTF8( tr.word() );
+	  vector<string> v = TiCC::split_at_first_of( tr.assignedTag(), "(,)" );
+	  if ( v.empty() ){
 	    throw runtime_error( "error: tag not in right format " );
 	  }
 	  string head = v[0];
@@ -222,10 +217,9 @@ void Test( istream& in ){
 	}
       }
       else {
-	vector<string> parts;
-	TiCC::split( s, parts );
+	vector<string> parts = TiCC::split( s );
 	for ( auto const& w : parts ){
-	  UnicodeString uWord = folia::UTF8ToUnicode(w);
+	  icu::UnicodeString uWord = TiCC::UnicodeFromUTF8(w);
 	  uWord.toLower();
 	  myMbma.Classify( uWord );
 	  myMbma.assign_compounds();
@@ -257,7 +251,7 @@ void Test( istream& in ){
 
 int main(int argc, char *argv[]) {
   std::ios_base::sync_with_stdio(false);
-  cerr << "mbma " << VERSION << " (c) CLTS, ILK 2014 - 2017" << endl;
+  cerr << "mbma " << VERSION << " (c) CLTS, ILK 2014 - 2018" << endl;
   cerr << "CLST  - Centre for Language and Speech Technology,"
        << "Radboud University" << endl
        << "ILK   - Induction of Linguistic Knowledge Research Group,"
