@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -29,12 +29,11 @@
 
 */
 
+#include "frog/ner_tagger_mod.h"
+
 #include <string>
 #include <iostream>
-#include <sstream>
-#include <fstream>
 #include <vector>
-#include <map>
 
 #include "config.h"
 #include "ticcutils/LogStream.h"
@@ -42,21 +41,19 @@
 #include "ticcutils/CommandLine.h"
 #include "ticcutils/PrettyPrint.h"
 #include "libfolia/folia.h"
-#include "frog/ucto_tokenizer_mod.h"
 #include "mbt/MbtAPI.h"
-#include "frog/ner_tagger_mod.h"
+#include "frog/ucto_tokenizer_mod.h"
 
 using namespace std;
 using namespace Timbl;
-using namespace TiCC;
 using namespace Tagger;
 
-LogStream my_default_log( cerr, "", StampMessage ); // fall-back
-LogStream *theErrLog = &my_default_log;  // fill the externals
+TiCC::LogStream my_default_log( cerr, "", StampMessage ); // fall-back
+TiCC::LogStream *theErrLog = &my_default_log;  // fill the externals
 
 vector<string> fileNames;
 
-Configuration configuration;
+TiCC::Configuration configuration;
 static string configDir = string(SYSCONF_PATH) + "/" + PACKAGE + "/";
 static string configFileName = configDir + "frog.cfg";
 
@@ -136,7 +133,6 @@ bool init(){
 }
 
 void Test( istream& in ){
-  string line;
   while ( in.good() ){
     string sentence = tokenizer.tokenizeStream( in );
     if ( sentence.empty() ){
@@ -154,7 +150,7 @@ void Test( istream& in ){
 
 int main(int argc, char *argv[]) {
   std::ios_base::sync_with_stdio(false);
-  cerr << "ner " << VERSION << " (c) CLTS, ILK 2014 - 2017" << endl;
+  cerr << "ner " << VERSION << " (c) CLTS, ILK 2014 - 2018" << endl;
   cerr << "CLST  - Centre for Language and Speech Technology,"
        << "Radboud University" << endl
        << "ILK   - Induction of Linguistic Knowledge Research Group,"
