@@ -1,6 +1,6 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  Copyright (c) 2006 - 2017
+  Copyright (c) 2006 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
 
@@ -29,22 +29,23 @@
 
 */
 
-#include <map>
-#include <string>
-#include "timbl/TimblAPI.h"
-#include "frog/Frog.h"
-#include "ticcutils/Configuration.h"
-#include "ticcutils/FileUtils.h"
 #include "frog/ucto_tokenizer_mod.h"
 
+#include <string>
+#include "timbl/TimblAPI.h"
+#include "frog/Frog-util.h"
+#include "ticcutils/Configuration.h"
+#include "ticcutils/FileUtils.h"
+#include "ticcutils/PrettyPrint.h"
+
 using namespace std;
-using namespace TiCC;
+using TiCC::operator<<;
 
-#define LOG *Log(uctoLog)
+#define LOG *TiCC::Log(uctoLog)
 
-UctoTokenizer::UctoTokenizer(LogStream * logstream) {
+UctoTokenizer::UctoTokenizer( TiCC::LogStream * logstream ) {
   tokenizer = 0;
-  uctoLog = new LogStream( logstream, "tok-" );
+  uctoLog = new TiCC::LogStream( logstream, "tok-" );
 }
 
 string resolve_configdir( const string& rules_name, const string& dir ){
@@ -59,7 +60,7 @@ string resolve_configdir( const string& rules_name, const string& dir ){
   return rules_name;
 }
 
-bool UctoTokenizer::init( const Configuration& config ){
+bool UctoTokenizer::init( const TiCC::Configuration& config ){
   if ( tokenizer )
     throw runtime_error( "ucto tokenizer is already initialized" );
   tokenizer = new Tokenizer::TokenizerClass();
@@ -78,8 +79,7 @@ bool UctoTokenizer::init( const Configuration& config ){
   string languages = config.lookUp( "languages", "tokenizer" );
   vector<string> language_list;
   if ( !languages.empty() ){
-    split_at( languages, language_list, "," );
-    using TiCC::operator<<;
+    language_list = TiCC::split_at( languages, "," );
     LOG << "Language List ="  << language_list << endl;
   }
   if ( tokenizer->getPassThru() ){
@@ -111,10 +111,12 @@ bool UctoTokenizer::init( const Configuration& config ){
       }
     }
   }
-
+  string textredundancy = config.lookUp( "textredundancy", "tokenizer" );
+  if ( !textredundancy.empty() ){
+    tokenizer->setTextRedundancy( textredundancy );
+  }
   tokenizer->setEosMarker( "" );
   tokenizer->setVerbose( false );
-  tokenizer->setSentenceDetection( true ); //detection of sentences
   tokenizer->setParagraphDetection( false ); //detection of paragraphs
   tokenizer->setXMLOutput( true );
   return true;
@@ -152,10 +154,19 @@ void UctoTokenizer::setInputEncoding( const std::string & enc ){
     throw runtime_error( "ucto tokenizer not initialized" );
 }
 
-void UctoTokenizer::setTextClass( const std::string& cls ){
+void UctoTokenizer::setInputClass( const std::string& cls ){
   if ( tokenizer ){
     if ( !cls.empty() )
-      tokenizer->setTextClass( cls );
+      tokenizer->setInputClass( cls );
+  }
+  else
+    throw runtime_error( "ucto tokenizer not initialized" );
+}
+
+void UctoTokenizer::setOutputClass( const std::string& cls ){
+  if ( tokenizer ){
+    if ( !cls.empty() )
+      tokenizer->setOutputClass( cls );
   }
   else
     throw runtime_error( "ucto tokenizer not initialized" );
@@ -178,6 +189,14 @@ void UctoTokenizer::setInputXml( bool b ){
     throw runtime_error( "ucto tokenizer not initialized" );
 }
 
+void UctoTokenizer::setTextRedundancy( const string& tr ) {
+  if ( tokenizer ){
+    tokenizer->setTextRedundancy( tr );
+  }
+  else
+    throw runtime_error( "ucto tokenizer not initialized" );
+}
+
 void UctoTokenizer::setPassThru( const bool b ) {
   if ( tokenizer ){
     tokenizer->setPassThru( b );
